<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2009 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik and der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

set_include_path(get_include_path() . PATH_SEPARATOR . dirname(__FILE__));
require_once 'Zend/Search/Lucene.php';

class WoogleIndexServiceNative extends WoogleIndexService {
	private $reIndexCount = 0;
	
	public function checkIndexService(){
		WoogleUtil::profileIn(__METHOD__);
		$r = false;
		$r = is_writable(WoogleLucene::getIndexPath());
		return WoogleUtil::profileOut(__METHOD__, $r);
	}
	
	public function getLocation() {
		WoogleUtil::profileIn(__METHOD__);
		return WoogleUtil::profileOut(__METHOD__, WoogleLucene::getIndexPath() . ' (Zend_Search_Lucene index)' );
	}
		
	public function numDocs() {
		WoogleUtil::profileIn(__METHOD__);
		return WoogleUtil::profileOut(__METHOD__, WoogleLucene::getIndexSize());
	}
	
	
	public function exists($id) {
		return WoogleLucene::exists($id);
	}
	
	public function getIndexFileSize() {
		WoogleUtil::profileIn(__METHOD__);
		WoogleUtil::profileOut(__METHOD__);
		return WoogleLucene::getIndexFileSize();
	}
	
	protected function startReIndex(){
		WoogleUtil::profileIn(__METHOD__);
		WoogleUtil::info('Starting ReIndex using NativeQS');
		$this->reIndexCount = 0;
		WoogleLucene::newIndex();
		WoogleUtil::profileOut(__METHOD__);
	}
	
	protected function stopReIndex(){
		WoogleUtil::profileIn(__METHOD__);
		WoogleUtil::info('Stopping ReIndex using NativeQS - added ' . $this->reIndexCount . ' documents');
		$this->optimizeIndex();
		$this->reIndexCount = 0;
		WoogleUtil::profileOut(__METHOD__);
	}
	
	protected function optimizeIndex(){
		WoogleUtil::profileIn(__METHOD__);
		WoogleLucene::optimizeIndex();
		WoogleUtil::profileOut(__METHOD__);
	}
	
	public function commitIndex(){
		WoogleUtil::profileIn(__METHOD__);
		WoogleLucene::commitIndex();
		WoogleUtil::profileOut(__METHOD__);
	}
	
	public function doAddItem($wd, $flush=false) {
		WoogleUtil::profileIn(__METHOD__);
		WoogleLucene::addItem($wd);
		WoogleUtil::profileOut(__METHOD__);
	}
	
	public function doAddItems($wda, $flush=true, $optimize=false, $printStatus=false){
		WoogleUtil::profileIn(__METHOD__);
		if ($printStatus) echo "Writing " . count($wda) . " pages to the index";
		foreach($wda as $wd){
			if ($printStatus) echo ".";
			WoogleLucene::addItem($wd);
		}
		$this->commitIndex();
		WoogleUtil::profileOut(__METHOD__);
	}
	
	public function doUpdateItem($wd) {
		WoogleUtil::profileIn(__METHOD__);
		WoogleLucene::deleteItem($wd->getShortId());
		WoogleLucene::addItem($wd);
		WoogleUtil::profileOut(__METHOD__);
	}
	public function doDeleteItem($id) {
		WoogleUtil::profileIn(__METHOD__);
		WoogleLucene::deleteItem($id);
		WoogleUtil::profileOut(__METHOD__);
	}
	
}

?>