<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2009 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik and der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

class WoogleQueryManagerNative implements WoogleQueryManager {
	private static $recencyDivider = 86400; // 60 * 60 * 24 (one day)
	private static $avgRecency;
	
	/**
	 * Logs query to the database
	 * 
	 * Note: WoogleQueryServiceNative ensures that this method is only called for the inital
	 * execution of a query. Browsing further result pages will call handleBrowse() instead
	 * 
	 * @param $searchString			raw query string
	 * @param $numHits				number of results for the query
	 * @return						void
	 */
	public function handleQuery($searchString, $numHits) {
		WoogleUtil::profileIn(__METHOD__);
		
		//
		// normalize searchString: sort words alphabetically if it was just a simple search with multiple terms
		//
/*		if ($query instanceof Zend_Search_Lucene_Search_Query_Boolean) {
			$subQueries = $query->getSubqueries();
			$cleanQ = true;
			foreach ($subQueries as $subQ) {
				if (!($subQ instanceof Zend_Search_Lucene_Search_Query_Term)) {
					$cleanQ = false;
					break;
				}
			}
			if ($cleanQ) {
				$terms = array();
				foreach ($subQueries as $t) {
					$terms[] = $t->getTerm()->text;
				}
				sort($terms);
				$searchString = implode(' ', $terms);
			}
		}*/
		
		//normalize search string to lowercase
		$searchString = self::normalizeQueryString($searchString);
		$hash = self::getHash($searchString);

		// prepare sql query
		$db = WoogleMWUtil::getDB();
		extract($db->tableNames('woogle_aggquery'));
		$sql = 'SELECT aggquery_id, aggquery_queryusers, aggquery_publicusers, aggquery_lastuserhash, aggquery_lastdate ' .
			'FROM ' . $woogle_aggquery . ' ' .
			'WHERE aggquery_qid = "' . mysql_real_escape_string($hash) . '"';
		$res = $db->query($sql);
		$sqlend = '';

		// hash to discover repeat queries
		$xhash = self::getHash(WoogleUtil::getAnonUserID() . $searchString);
		
		// check if database entry for given query exists
		if ($row = $db->fetchObject($res)) {
			// existing database entry for given query - needs to be updated now!  
			WoogleUtil::debug("Found database log entry for query " . $searchString);
			
			// set values
			$id 				= $row->aggquery_id;
			$needexec 			= 'aggquery_needexec + 1';		// increment needexec by 1

			// only increment click if last click by user was > 60 sec away
			if ( ($xhash == $row->aggquery_lastuserhash) && ( (time() - $row->aggquery_lastdate) < 61)){
				// repeat search - do nothing...
				WoogleUtil::debug('repeat - do nothing: '  . ( time() - $row->aggquery_lastdate )); 
			} else {
				$needexecclick 		= 'aggquery_needexecclick + 1'; // increment neexec without clicks (decrement later on first click)
				WoogleUtil::debug('increment: '  . ( $row->aggquery_lastdate - time() )); 
			}
			
			// only increment click if last click by user was > 60 sec away
			if (($xhash == $row->aggquery_lastuserhash) && (($row->aggquery_lastdate - time()) < 61)){
				// repeat search - do nothing...
			} else {
				$needexecclick 		= 'aggquery_needexecclick + 1'; // increment neexec without clicks (decrement later on first click)
			}
			
			$recency 			= $this->calcRecency(true);		// TODO update recency
			$needexecavgpage 	= '((aggquery_needexec - 1) * aggquery_needexecavgpage + 1) / aggquery_needexec';
			
			$sql = 'UPDATE';
			// update list of query users
			if (($users = $this->ensureCurrentUser(WoogleUtil::getAnonUserID(), $row->aggquery_queryusers)) !== true) {
				$sqlend .= ', aggquery_sizequeryusers	= ' . count(explode(';', $users));	
						// increment size of list (don't do ++ as update is not synchronous)
				$sqlend .= ', aggquery_queryusers		= "' . mysql_real_escape_string($users) . '"';
						// update list
						}
						
			// TODO dummy
/*			if (($users = $this->ensureCurrentUser($row->aggquery_publicusers)) !== true) {
				$sqlend .= ', aggquery_publicusers	= "' . mysql_real_escape_string($users) . '"';
			}*/
			$sqlend .= ' WHERE aggquery_id = ' . $row->aggquery_id;
			
		} else {
			// no database entry for given query existing - needs to be created now!
			WoogleUtil::debug("Creating new database log entry for query " . $searchString);
			
			// initial values
			$needexec 			= 1;
			$needexecclick 		= 1;
			$recency 			= $this->calcRecency();
			$needexecavgpage 	= 1;
			
			$sql = 'INSERT INTO';
			$sqlend = 
				', aggquery_qid				= "' . mysql_real_escape_string($hash) . '"' .
				', aggquery_needstring		= "' . mysql_real_escape_string($searchString) . '"' .
				', aggquery_firstdate		= ' . time() .
				', aggquery_sizequeryusers	= 1' .
				', aggquery_needexecpages	= 0' .
				', aggquery_queryusers		= "' . mysql_real_escape_string($this->ensureCurrentUser(WoogleUtil::getAnonUserID())) . '"' .
				', aggquery_publicusers		= "' . mysql_real_escape_string($this->ensureCurrentUser('dummy')) . '"'; // TODO dummy

			
		}
		
		// update general values
		$sql .= ' ' . $woogle_aggquery . ' SET ' .
			'aggquery_needexec			= ' . $needexec . ', ';
		if (isset($needexecclick)) $sql .= 			'aggquery_needexecclick		= ' . $needexecclick . ', ';
		$sql .=
			'aggquery_recency			= ' . $recency . ', ' .
			'aggquery_lastdate			= ' . time() . ', ' .
			'aggquery_resulthits		= ' . mysql_real_escape_string($numHits) . ', ' .
			'aggquery_needexecavgpage	= ' . $needexecavgpage . ', ' .
			'aggquery_lastuserhash		= "' . mysql_real_escape_string($xhash) . '"' .
			$sqlend;
			
		$db->query($sql);
		$db->commit();
		
		WoogleUtil::profileOut(__METHOD__);
	}
	

	
	/**
	 * Log additional executions of a given query (browsing results)
	 * 
	 * @param	$searchString		String, query string
	 * @param	$sessionHash		String, user session hash
	 * @return						void
	 */
	public function handleBrowse($searchString, $sessionHash) {
		WoogleUtil::profileIn(__METHOD__);
		
		$searchString = self::normalizeQueryString($searchString);
		$hash = self::getHash($searchString);

		// retrieve data for query from db (should exist already)
		$db = WoogleMWUtil::getDB();
		extract($db->tableNames('woogle_aggquery', 'woogle_aggquery_session'));
		$sql = 'SELECT aggquery_id ' .
			'FROM ' . $woogle_aggquery . ' ' .
			'WHERE aggquery_qid = "' . mysql_real_escape_string($hash) . '"';
		$res = $db->query($sql);
		
		if ($row = $db->fetchObject($res)) {
			WoogleUtil::debug("Logging browse action for query " . $searchString);

			$id = $row->aggquery_id;
			$sessionExists = $this->checkSession($sessionHash, $woogle_aggquery_session);
			
			$sql = 'UPDATE ' . $woogle_aggquery . ' SET ' .
				(!$sessionExists ?
				'aggquery_needexecpages		= aggquery_needexecpages + 1, ' : ''
				) .
				'aggquery_needexecavgpage	= (aggquery_needexec * aggquery_needexecavgpage + 1) / aggquery_needexec ' .
				'WHERE aggquery_id = ' . $id;
			$db->query($sql);
			$db->commit();
		} else {
			// do nothing, as initial query could not be found and therefore not be updated
			WoogleUtil::error("Did not find query log entry for query " . $searchString);
		}
		WoogleUtil::profileOut(__METHOD__);
	}
	
	/**
	 * Log user result click to database
	 * 
	 * @param $searchString		query string for which the result was displayed
	 * @param $url				result url
	 * @param $position			position of result in result list
	 * @param $session			user session hash
	 * @return					void
	 */
	public function handleClick($searchString, $url, $position, $session) {
		WoogleUtil::profileIn(__METHOD__);
		
		$searchString = self::normalizeQueryString($searchString);
		$hash = self::getHash($searchString);
		
		//
		// Update click log
		//		
		
		// check if query/click pair already exists
		$clickExists = false;
		$db = WoogleMWUtil::getDB();
		extract($db->tableNames('woogle_aggclick', 'woogle_aggquery', 'woogle_aggclick_session'));
		$sql = 'SELECT aggclick_id ' .
			'FROM ' . $woogle_aggclick . ' ' .
			'WHERE aggclick_qid = "' . mysql_real_escape_string($hash) . '" ' .
			'AND aggclick_subject = "' . mysql_real_escape_string($url) . '"';
		$res = $db->query($sql);

		if ($row = $db->fetchObject($res)) {
			//
			// query/click pair exists -> update
			//
			WoogleUtil::debug("Found click log entry for query " . $searchString . " and result " . $url);
			$clickExists = true;
			
			$clicks 	= 'aggclick_clicks + 1';
			$recency 	= $this->calcRecency(true, 'aggclick_recency', 'aggclick_clicks');
			$sql 		= 'UPDATE';
			$sqlend 	= ' WHERE aggclick_id = ' . $row->aggclick_id;
			
		} else {
			//
			// query/click pair exists -> create
			//
			WoogleUtil::debug("Did not find click log entry for query " . $searchString . " and result " . $url);
			$clickExists = false;
			
			// initial values
			$clicks = 1;
			$recency = $this->calcRecency();
			$sql = 'INSERT INTO';
			$sqlend =
				', aggclick_qid			= "' . mysql_real_escape_string($hash) . '"' .
				', aggclick_subject		= "' . mysql_real_escape_string($url) . '"' .
				', aggclick_firstdate	= ' . time();
		}
		
		// general values
		$sql .= ' ' . $woogle_aggclick . ' SET ' .
			'aggclick_clicks	= ' . $clicks . ', ' .
			'aggclick_position	= ' . mysql_real_escape_string($position) . ', ' .
			'aggclick_recency	= ' . $recency . ', ' .
			'aggclick_lastdate	= ' . time() .
			$sqlend;
		$db->query($sql);
		WoogleUtil::debug('Updated click log: ' . $sql);
		
		
		//
		// Update query log after click
		//		
		
		// check if session exists
		$sessionExists = $this->checkSession($session, $woogle_aggclick_session);
		
		$sql = 'UPDATE ' . $woogle_aggquery . ' SET ' .
			// decrease needexecclick if this is the first click in the session
			(!$sessionExists ? 
			'aggquery_needexecclick		= aggquery_needexecclick - 1, ' : ''
			) .
			// increase uniqueresults if this was the first click on this result for the given query
			(!$clickExists ?
			'aggquery_uniqueresults		= aggquery_uniqueresults + 1, ' : ''
			) .
			// always increase resultclicks
			'aggquery_resultclicks		= aggquery_resultclicks + 1 ' .
			'WHERE aggquery_qid = "' . mysql_real_escape_string($hash) . '"';
		$db->query($sql);
		$db->commit();
		WoogleUtil::profileOut(__METHOD__);
	}

	
	public function getUnsatisfiedQueriesWithFewHits($number) {
		WoogleUtil::profileIn(__METHOD__);
		$ret = array();
		
		$db = WoogleMWUtil::getDB();
		extract($db->tableNames('woogle_aggquery'));

		$sql = "SELECT *, (1.0 * aggquery_needexecclick/aggquery_needexec) AS test, ((1.0 * aggquery_needexecclick/aggquery_needexec) + (1- 1.0 * aggquery_resultclicks/aggquery_needexec)) AS val FROM " .
		 		$woogle_aggquery . " ORDER BY val DESC LIMIT " . $number;
		
		$res = $db->query($sql);
		if ($row = $db->fetchObject($res)) {
			$r =  array(
				'queryId'				=> $row->aggquery_qid,
				'needString'			=> $row->aggquery_needstring,
				'localNDFUsers'			=> self::getUsers($row->aggquery_queryusers),	// TODO undefined for Woogle (IS only)
				'queryUsers'			=> self::getUsers($row->aggquery_queryusers), 
			);
			array_push($ret, $r);
		}
		
		WoogleUtil::profileOut(__METHOD__);
		return $ret;
	}
	
	public function getUnsatisfiedQueriesWithManyDifferentClicks($number) {
		WoogleUtil::profileIn(__METHOD__);
		$ret = array();
		
		$db = WoogleMWUtil::getDB();
		extract($db->tableNames('woogle_aggquery'));

		$sql = 'SELECT *, (1.0*aggquery_resultclicks/aggquery_needexec) AS val FROM ' . $woogle_aggquery . ' ORDER BY val DESC LIMIT ' . $number;
		
		$res = $db->query($sql);
		if ($row = $db->fetchObject($res)) {
			$r =  array(
				'queryId'				=> $row->aggquery_qid,
				'needString'			=> $row->aggquery_needstring,
			);
			array_push($ret, $r);
		}
		
		WoogleUtil::profileOut(__METHOD__);
		return $ret;
	}
	
	public function getUnsatisfiedQueriesWithMuchBrowsing($number) {
		WoogleUtil::profileIn(__METHOD__);
		$ret = array();
		
		$db = WoogleMWUtil::getDB();
		extract($db->tableNames('woogle_aggquery'));

		$sql = 'SELECT *,(1.0*aggquery_needexecpages/aggquery_needexec*aggquery_needexecavgpage) AS val FROM ' . $woogle_aggquery . ' ORDER BY val DESC LIMIT ' . $number;
		
		$res = $db->query($sql);
		if ($row = $db->fetchObject($res)) {
			$r =  array(
				'queryId'				=> $row->aggquery_qid,
				'needString'			=> $row->aggquery_needstring,
			);
			array_push($ret, $r);
		}
		
		WoogleUtil::profileOut(__METHOD__);
		return $ret;
	}
	
	public function getUnsatisfiedQueriesAggregations($number) {
		WoogleUtil::profileIn(__METHOD__);
		$ret = array();
				
		// TODO - really required?
		
		/*
		 
		 	// UnsatisfiedQueries are queries with 0 hits
	public List<InformationNeedSet> getUnsatisfiedQueriesAggregations(int number) {
		List<InformationNeedSet> res = new ArrayList<InformationNeedSet>(); 
		try {
			String unsatisfiedQueries = "SELECT * FROM " + QT.table + " WHERE " + QT.resulthits + " = 0 ORDER BY " + QT.sizequeryusers +" LIMIT " + number;
			log.debug("getUnsatisfiedQueriesAggregations SQL STATEMENT : \n" + unsatisfiedQueries +"\n");
			ResultSet rs = this.getStatement().executeQuery(unsatisfiedQueries);
			while( rs.next() ) {
				InformationNeedSet ins = new InformationNeedSet();
				ins.setLabel(rs.getString(QT.needstring));
				InformationNeed in = new InformationNeed();
				in.setPublicQueryUsers(QueryManager.fromCSV(rs.getString(QT.queryusers)));
				in.setLastDate(new Date(rs.getLong(QT.lastdate)));
				in.setFirstDate(new Date(rs.getLong(QT.firstdate)));
				in.setFreq(rs.getInt(QT.needexec));
				in.setNeedString(rs.getString(QT.needstring));
				ins.addInformationNeed(in);
				res.add(ins);
			}
		} catch (SQLException e) {
			log.error(e.getMessage(),e); 
		}
		return res;
	}
		 */
		
		WoogleUtil::profileOut(__METHOD__);
		return $ret;
	}
	
	public function getInformationNeed($searchString) {
		WoogleUtil::profileIn(__METHOD__);
		$ret = false;
		
		$qStart = microtime(true);
		
		$searchString = self::normalizeQueryString($searchString);
		
		$db = WoogleMWUtil::getDB();
		extract($db->tableNames('woogle_aggquery'));
		
//		echo "\n table name: " . $woogle_aggquery . "\n";
//		echo "\n qhash for " . $searchString . " : " . self::getHash($searchString) . "\n";
		
		$sql = 'SELECT aggquery_qid, aggquery_needstring, aggquery_needexec, aggquery_needexecclick, aggquery_recency, ' .
			'aggquery_resulthits, aggquery_firstdate, aggquery_lastdate, aggquery_sizequeryusers, aggquery_queryusers, ' .
			'aggquery_needexecpages, aggquery_needexecavgpage, aggquery_resultclicks, aggquery_uniqueresults ' .
			'FROM ' . $woogle_aggquery . ' ' .
			'WHERE aggquery_qid = "' . mysql_real_escape_string(self::getHash($searchString)) . '"';
		
		$res = $db->query($sql);
		
		if ($row = $db->fetchObject($res)) {
//			echo "\n QID " . $row->aggquery_qid . "  \n";
			$ret = array(
				'needString'									=> $row->aggquery_needstring, 
				'needExecutions'								=> $row->aggquery_needexec,
				'needExecutionsWithoutResultClick'				=> $row->aggquery_needexecclick,
				'recency'										=> $row->aggquery_recency * self::$recencyDivider,
				'resultHits'									=> $row->aggquery_resulthits,
				'firstDate'										=> $row->aggquery_firstdate,
				'lastDate'										=> $row->aggquery_lastdate,
				'sizeQueryUsers'								=> $row->aggquery_sizequeryusers,
				'publicQueryUsers'								=> self::getUsers($row->aggquery_queryusers),
				'needExecutionsWithFurtherPages'				=> $row->aggquery_needexecpages,
				'needExecutionsWithFurtherPagesAveragePages'	=> $row->aggquery_needexecavgpage,
				'resultClicks'									=> $row->aggquery_resultclicks,
				'uniqueClickedResults'							=> $row->aggquery_uniqueresults,
			);
		} 
		
		// do some logging
		$qStop = microtime(true);
		$qTime = $qStop - $qStart;
		WoogleUtil::perflog($qTime . ';' . 'infneed', 'querymetadata');
		WoogleUtil::profileOut(__METHOD__);
		return $ret;
	}
	
	public function getRelatedQueries($searchString, $max, $excludeQuery) {
		WoogleUtil::profileIn(__METHOD__);
		$qStart = microtime(true);
		$queries = array();
		
		$searchString = self::normalizeQueryString($searchString);
		
		$db = WoogleMWUtil::getDB();
		extract($db->tableNames('woogle_aggquery'));
		$sql = 'SELECT aggquery_needstring, aggquery_needexec, aggquery_needexecclick, aggquery_recency, ' .
			'aggquery_resulthits, aggquery_firstdate, aggquery_lastdate, aggquery_sizequeryusers, aggquery_queryusers, ' .
			'aggquery_needexecpages, aggquery_needexecavgpage, aggquery_resultclicks, aggquery_uniqueresults ' .
			'FROM ' . $woogle_aggquery . ' ' .
			'WHERE aggquery_needstring LIKE "%' . mysql_real_escape_string($searchString) . 
			'%" LIMIT ' . $max;
		$res = $db->query($sql);
		
		WoogleUtil::debug('Found ' . $res->numRows() . ' related queries for ' . $sql); 
		
		while ($row = $db->fetchObject($res)) {
			$q = array(
				'needString'									=> $row->aggquery_needstring, 
				'needExecutions'								=> $row->aggquery_needexec,
				'needExecutionsWithoutResultClick'				=> $row->aggquery_needexecclick,
				'recency'										=> $row->aggquery_recency * self::$recencyDivider,
				'resultHits'									=> $row->aggquery_resulthits,
				'firstDate'										=> $row->aggquery_firstdate,
				'lastDate'										=> $row->aggquery_lastdate,
				'sizeQueryUsers'								=> $row->aggquery_sizequeryusers,
				'publicQueryUsers'								=> self::getUsers($row->aggquery_queryusers),
				'needExecutionsWithFurtherPages'				=> $row->aggquery_needexecpages,
				'needExecutionsWithFurtherPagesAveragePages'	=> $row->aggquery_needexecavgpage,
				'resultClicks'									=> $row->aggquery_resultclicks,
				'uniqueClickedResults'							=> $row->aggquery_uniqueresults,
			);
			
			// check exclude query
			if (!($excludeQuery) || ($q['needString'] != $searchString)){		
				array_push($queries, $q);
			}
		}
		
		// do some logging
		$qStop = microtime(true);
		$qTime = $qStop - $qStart;
		WoogleUtil::perflog($qTime . ';' . 'related', 'querymetadata');
		return WoogleUtil::profileOut(__METHOD__, $queries);
	}
	
	// TODO test
	public function getPopularQueries($number) {
		WoogleUtil::profileIn(__METHOD__);

		$queries = array();
		
		$db = WoogleMWUtil::getDB();
		extract($db->tableNames('woogle_aggquery'));
		
		$sql = 'SELECT aggquery_needstring, aggquery_needexec ' .
			'FROM ' . $woogle_aggquery . ' ' .
			' ORDER BY aggquery_needexec DESC ' . 
			' LIMIT ' . $number;
		$res = $db->query($sql);
		
		WoogleUtil::debug('Found ' . $res->numRows() . ' related queries for ' . $sql); 
				
		while ($row = $db->fetchObject($res)) {
			$q = new WoogleTermFreq($row->aggquery_needstring,$row->aggquery_needexec,$row->aggquery_needstring);
			/*
			$q = array(
				'name'					=> $row->aggquery_needstring, 
				'entity'				=> $row->aggquery_needstring,
				'freq'					=> $row->aggquery_needexec,
			);*/
			array_push($queries, $q);
		}
	
		WoogleUtil::profileOut(__METHOD__);
		return $queries;
	}
	

	public function getPopularResults($number) {
		WoogleUtil::profileIn(__METHOD__);

		$results = array();
		
		$db = WoogleMWUtil::getDB();
		extract($db->tableNames('woogle_aggclick'));
		
		$sql = 'SELECT aggclick_subject, sum(aggclick_clicks) AS sclicks ' .
			'FROM ' . $woogle_aggclick . ' ' .
			' GROUP BY aggclick_subject DESC ' . 
			' ORDER BY sclicks DESC ' . 
			' LIMIT ' . $number;
		$res = $db->query($sql);
		
		WoogleUtil::debug('Found ' . $res->numRows() . ' related queries for ' . $sql); 
				
		while ($row = $db->fetchObject($res)) {
			$ef = new WoogleEntityFreq(array('uri' => $row->aggclick_subject),$row->sclicks);
			/*
			$ka = array(
				'id'					=> $row->aggclick_subject, 
				'uri'					=> $row->aggclick_subject,
				'label'					=> $row->aggclick_subject,
			);
			
			$ef = array(
				'freq'					=> $row->sclicks,
				'entity'				=> $ka,
			);*/
			array_push($results, $ef);
		}
		
		WoogleUtil::profileOut(__METHOD__);
		return $results;
	}
	
	/**
	 * List of popular repositories (does not make so much sense
	 * for WoogleNative who basically has itself as a repository only 
	 *
	 * @see includes/WoogleQueryManager#getPopularSources()
	 */
	public function getPopularSources($number) {
		WoogleUtil::profileIn(__METHOD__);

		$results = array();
		
		$db = WoogleMWUtil::getDB();
		extract($db->tableNames('woogle_aggclick'));
		
		$sql = 'SELECT aggclick_repoid, sum(aggclick_clicks) AS sclicks ' .
			'FROM ' . $woogle_aggclick . ' ' .
			' GROUP BY aggclick_repoid DESC ' . 
			' ORDER BY sclicks DESC ' . 
			' LIMIT ' . $number;
		$res = $db->query($sql);
		
		WoogleUtil::debug('Found ' . $res->numRows() . ' related queries for ' . $sql); 
				
		while ($row = $db->fetchObject($res)) {
			$ef = new WoogleEntityFreq(array('repoId' => $row->aggclick_repoid),$row->sclicks);
			/*
			$ka = array(
				'repoId'				=> $row->aggclick_repoid, 
			);
			
			$ef = array(
				'freq'					=> $row->sclicks,
				'entity'				=> $ka,
			);*/
			array_push($results, $ef);
		}
		
		WoogleUtil::profileOut(__METHOD__);
		return $results;
	}
	
	// TODO Check URI?
	public function getTags($uri) {
		WoogleUtil::profileIn(__METHOD__);

		$results = array();
		
		$db = WoogleMWUtil::getDB();
		extract($db->tableNames('woogle_aggclick', 'woogle_aggquery'));
		
		$sql = 'SELECT qm.aggquery_needstring AS aggquery_needstring, ' .
				'cqa.aggclick_clicks AS sclicks ' . 
				'FROM ' . $woogle_aggquery . ' qm INNER JOIN ' . $woogle_aggclick . ' cqa ON qm.aggquery_qid = cqa.aggclick_qid ' .
				'WHERE cqa.aggclick_subject = "' . $uri . '" ' . 
				' ORDER BY sclicks DESC'; 
		$res = $db->query($sql);
		
		WoogleUtil::debug('Found ' . $res->numRows() . ' rows for ' . $sql); 
				
		while ($row = $db->fetchObject($res)) {
			$t = new WoogleTermFreq($row->aggquery_needstring,$row->sclicks,$row->aggquery_needstring);
			/*$t = array(
				'freq'					=> $row->sclicks,
				'entity'				=> $row->aggquery_needstring,
				'name'					=> $row->aggquery_needstring,
			);*/
			array_push($results, $t);
		}
		
		WoogleUtil::profileOut(__METHOD__);
		return $results;
	}
	
	public function getAvgPages() {
		WoogleUtil::profileIn(__METHOD__);
		$pagesAvg = $this->getField('AVG(aggquery_needexecpages)');
		if ($pagesAvg === false) {
			$pagesAvg = 0;
		}
		WoogleUtil::profileOut(__METHOD__);
		return $pagesAvg;
	}
	
	public function getAvgRecency() {
		WoogleUtil::profileIn(__METHOD__);
		if (empty(self::$avgRecency)) {
			$avgRecency = $this->getField('AVG(aggquery_recency)');
			if ($avgRecency === false) {
				$avgRecency = time();
			} else {
				$avgRecency *= self::$recencyDivider;
			}
			self::$avgRecency = $avgRecency;
		}
		WoogleUtil::profileOut(__METHOD__);
		return self::$avgRecency;
	}
	

	
	/**
	 * Ensure that current user is part of user list in $blob
	 * 
	 * @param $userId				String, userId
	 * @param $blob					existing list of users in database (comma-separated list)
	 * @return boolean or blob		either true (if user is already part of $blob, or new value for $blob		
	 */
	private function ensureCurrentUser($userId, $blob = '') {
		WoogleUtil::profileIn(__METHOD__);
		
		$currentUser 	= $userId;
		$users 			= self::getUsers($blob);			// get users in blob
		
		if (array_search($currentUser, $users) !== false) {
			// user id already exists in blob
			return WoogleUtil::profileOut(__METHOD__, true);
		} else {
			// user id does not yet exist in blob
			$newBlob = $blob . (empty($blob) ? '' : ';') . urlencode($currentUser);
			return WoogleUtil::profileOut(__METHOD__, $newBlob);
		}
	}
	
	/**
	 * Returns an avaraged integeger of days since unix epoch or sql expression
	 * 
	 * @param $calcNew		boolean, will use further parameters to calculate if set true
	 * @param $daysField	String, sql column name for recency value
	 * @param $numField		String, sql column name for number of executions
	 * @return int/string	recency value (averaged days since begin of unix epoch) or sql expression to calculate recency value
	 */
	private function calcRecency($calcNew = false, $daysField = 'aggquery_recency', $numField = 'aggquery_needexec') {
		WoogleUtil::profileIn(__METHOD__);

		// recencyDivider = one day
		$currentDays = intval(time() / self::$recencyDivider);	// days since begin of unix epoch (for today/now)
		
		if ($calcNew) {
			return WoogleUtil::profileOut(__METHOD__, '((' . $numField . ' - 1) * ' . $daysField . ' + ' . $currentDays . ') / ' . $numField);
			/*
			 * Formula is like:
			 * 
			 * 		newRecency = ( (needExecutions-1) * dbRecency + currentRecency ) / needExecutions 
			 * 
			 */
		} else {
			return WoogleUtil::profileOut(__METHOD__, $currentDays);
		}
	}
	
	private function getField($name) {
		WoogleUtil::profileIn(__METHOD__);
		$dbr = WoogleMWUtil::getDB();
		$result = $dbr->selectField(
			'woogle_aggquery',
			$name,
			array(),
			__METHOD__
		);
		WoogleUtil::profileOut(__METHOD__);
		return $result;
	}
	
	/**
	 * Checks if a session for a given session key is existing.
	 * If not, it creates a new entry in the session table
	 * 
	 * @param $sessionHash		String, sessionHash key
	 * @param $table			String, session table name
	 * @return boolean			if session was existing or not
	 */
	private function checkSession($sessionHash, $table){
		
		$sessionExists = false;
		
		$db = WoogleMWUtil::getDB();
		
		$sql = 'SELECT session_hash ' .
			'FROM ' . $table . ' ' .
			'WHERE session_hash = "' . mysql_real_escape_string($sessionHash) . '"';
		$res = $db->query($sql);
		
		if ($row = $db->fetchObject($res)) {
			WoogleUtil::debug("Found session hash " . $sessionHash . " in db");
			$sessionExists = true;
		} else {
			WoogleUtil::debug("Did not find session hash " . $sessionHash . " in db");
			$sessionExists = false;
			// create session
			$sql = 'INSERT INTO ' . $table . ' ' .
				'SET session_hash = "' . mysql_real_escape_string($sessionHash) . '"';
			$db->query($sql);
		}
		
		return $sessionExists;
	}
	
	/**
	 * Convert CSV-string to array
	 * 
	 * @param $blob					String, with user ids separated by semicolon
	 * @return array of String		user id's
	 */
	public static function getUsers($blob = '') {
		WoogleUtil::profileIn(__METHOD__);
		
		// explode CSV list 
		$users = explode(';', $blob);
		
		// convert to array
		$cleanUsers = array();
		foreach ($users as $user) {
			$cleanUsers[] = urldecode($user);
		}
		return WoogleUtil::profileOut(__METHOD__, $cleanUsers);
	}
	
	/**
	 * drop db tables
	 * @return number of tables dropped
	 */
	public static function dropDb(){
		WoogleUtil::profileIn(__METHOD__);
		$count = 0;
		
		// noting to delete!
		if (!WoogleMWUtil::tableExists('woogle_aggquery')) return -1;

		global $wgDBprefix;
		$px = $wgDBprefix;
		if (!isset($wgDBprefix)) $px = '';
		
		$db = WoogleMWUtil::getDB();
		
		//extract($db->tableNames('woogle_query', 'woogle_click', 'woogle_aggquery', 'woogle_aggquery_session',
		//	'woogle_aggclick', 'woogle_aggclick_session'), EXTR_PREFIX_ALL, $px);
		extract($db->tableNames('woogle_query', 'woogle_click', 'woogle_aggquery', 'woogle_aggquery_session',
			'woogle_aggclick', 'woogle_aggclick_session'));
		$fname = __METHOD__;
		
		if ($db->tableExists($woogle_aggquery)) {
			$sql = 'DROP TABLE ' . $woogle_aggquery . ';';
			$db->query($sql, $fname);
			$count++;
		}
		if ($db->tableExists($woogle_aggquery_session)) {
			$sql = 'DROP TABLE ' . $woogle_aggquery_session . ';';
			$db->query($sql, $fname);
			$count++;
		}
		if ($db->tableExists($woogle_aggclick)) {
			$sql = 'DROP TABLE ' . $woogle_aggclick . ';';
			$db->query($sql, $fname);
			$count++;
		}
		if ($db->tableExists($woogle_aggclick_session)) {
			$sql = 'DROP TABLE ' . $woogle_aggclick_session . ';';
			$db->query($sql, $fname);
			$count++;
		}
		
		return WoogleUtil::profileOut(__METHOD__, $count);
	}
	
	
	/**
	 * create loggging tables
	 * @return number of tables created
	 */
	public static function initDb(){
		WoogleUtil::profileIn(__METHOD__);
		$count = 0;
		
		// noting to add..
		if (WoogleMWUtil::tableExists('woogle_aggquery')) return -1;
		
		global $wgDBTableOptions;
		$db = WoogleMWUtil::getDB();
		
		/*
		global $wgDBprefix;
		$px = $wgDBprefix;
		if (!isset($wgDBprefix)) $px = '';
		extract($db->tableNames('woogle_query', 'woogle_click', 'woogle_aggquery', 'woogle_aggquery_session',
			'woogle_aggclick', 'woogle_aggclick_session'), EXTR_PREFIX_ALL, $px);
		*/
		extract($db->tableNames('woogle_query', 'woogle_click', 'woogle_aggquery', 'woogle_aggquery_session',
			'woogle_aggclick', 'woogle_aggclick_session'));
		$fname = __METHOD__;

/*		// old query logging into db
		if ($db->tableExists($woogle_query) == false) {
			$sql = 'CREATE TABLE ' . $woogle_query . ' (' .
					'query_id		INTEGER		UNSIGNED	NOT NULL	AUTO_INCREMENT, ' .
					'query_time		INTEGER		UNSIGNED	NOT NULL, ' .
					'query_referer	VARCHAR(200)			NOT NULL, ' .
					'query_remoteip	VARCHAR(50)				NOT NULL, ' .
					'query_user		INTEGER		UNSIGNED	NOT NULL, ' .
					'query_query	VARCHAR(200)			NOT NULL, ' .
					'query_numhits	INTEGER		UNSIGNED	NOT NULL, ' .
					'query_count	INTEGER		UNSIGNED	NOT NULL, ' .
					'query_start	INTEGER		UNSIGNED	NOT NULL, ' .
					'PRIMARY KEY(query_id)) ' . $wgDBTableOptions;
			$db->query($sql, $fname);
		}
		if ($db->tableExists($woogle_click) == false) {
			$sql = 'CREATE TABLE ' . $woogle_click . ' (' .
					'click_id		INTEGER		UNSIGNED	NOT NULL	AUTO_INCREMENT, ' .
					'click_time		INTEGER		UNSIGNED	NOT NULL, ' .
					'click_referer	VARCHAR(200)			NOT NULL, ' .
					'click_remoteip	VARCHAR(50)				NOT NULL, ' .
					'click_user		INTEGER		UNSIGNED	NOT NULL, ' .
					'click_query	VARCHAR(200)			NOT NULL, ' .
					'click_position	INTEGER		UNSIGNED	NOT NULL, ' .
					'click_url		VARCHAR(200)			NOT NULL, ' .
					'PRIMARY KEY(click_id)) ' . $wgDBTableOptions;
			$db->query($sql, $fname);
		}*/
		
		// Table for aggregate query logging
		if ($db->tableExists($woogle_aggquery) == false) {
			$sql = 'CREATE TABLE ' . $woogle_aggquery . ' (' .
					'aggquery_id				INTEGER		UNSIGNED	NOT NULL	AUTO_INCREMENT, ' .
					'aggquery_qid				VARCHAR(200)			NOT NULL, ' .
					'aggquery_needstring		VARCHAR(200)  CHARACTER SET utf8 COLLATE utf8_bin NOT NULL, ' . // to avoid muller=müller (http://dev.mysql.com/doc/refman/5.1/en/charset-collation-effect.html)
					'aggquery_needexec			INTEGER		UNSIGNED	NOT NULL, ' .
					'aggquery_needexecclick		INTEGER		UNSIGNED	NOT NULL, ' .
					'aggquery_recency			FLOAT		UNSIGNED	NOT NULL, ' .
					'aggquery_firstdate			INTEGER		UNSIGNED	NOT NULL, ' .
					'aggquery_lastdate			INTEGER		UNSIGNED	NOT NULL, ' .
					'aggquery_resulthits		INTEGER		UNSIGNED	NOT NULL, ' .
					'aggquery_sizequeryusers	INTEGER		UNSIGNED	NOT NULL, ' .
					'aggquery_needexecpages		INTEGER		UNSIGNED	NOT NULL, ' .
					'aggquery_needexecavgpage	FLOAT		UNSIGNED	NOT NULL, ' .
					'aggquery_resultclicks		INTEGER		UNSIGNED	NOT NULL, ' .
					'aggquery_uniqueresults		INTEGER		UNSIGNED	NOT NULL, ' .
					'aggquery_queryusers		LONGBLOB				NOT NULL, ' .
					'aggquery_publicusers		LONGBLOB				NOT NULL, ' .
					'aggquery_lastuserhash		VARCHAR(200)			NOT NULL, ' .
					'PRIMARY KEY(aggquery_id), ' .
					'UNIQUE INDEX(aggquery_qid), ' . 
					'UNIQUE INDEX(aggquery_needstring)) ' . $wgDBTableOptions;
			$db->query($sql, $fname);
			$count++;
		}
		
		// Table for query user sessions
		if ($db->tableExists($woogle_aggquery_session) == false) {
			$sql = 'CREATE TABLE ' . $woogle_aggquery_session . ' (' .
					'session_hash		VARCHAR(200)			NOT NULL, ' .
					'UNIQUE INDEX(session_hash)) ' . $wgDBTableOptions;
			$db->query($sql, $fname);
			$count++;
		}
		
		// Table for aggregate result click logging
		if ($db->tableExists($woogle_aggclick) == false) {
			$sql = 'CREATE TABLE ' . $woogle_aggclick . ' (' .
					'aggclick_id		INTEGER		UNSIGNED	NOT NULL	AUTO_INCREMENT, ' .
					'aggclick_qid		VARCHAR(200)			NOT NULL, ' .
					'aggclick_subject	VARCHAR(200)			NOT NULL, ' .
					'aggclick_clicks	INTEGER		UNSIGNED	NOT NULL, ' .
					'aggclick_position	INTEGER		UNSIGNED	NOT NULL, ' .
					'aggclick_recency	FLOAT		UNSIGNED	NOT NULL, ' .
					'aggclick_firstdate	INTEGER		UNSIGNED	NOT NULL, ' .
					'aggclick_lastdate	INTEGER		UNSIGNED	NOT NULL, ' .
					'aggclick_repoid	INTEGER		UNSIGNED	NOT NULL, ' .
					'PRIMARY KEY(aggclick_id), ' .
					'UNIQUE INDEX(aggclick_subject, aggclick_qid), ' . // additional index since mysql can only work with leftmost indexes (http://dev.mysql.com/doc/refman/5.1/en/mysql-indexes.html)  
					'UNIQUE INDEX(aggclick_qid, aggclick_subject)) ' . $wgDBTableOptions;
			$db->query($sql, $fname);
			$count++;
		}
		
		// Table for click user sessions
		if ($db->tableExists($woogle_aggclick_session) == false) {
			$sql = 'CREATE TABLE ' . $woogle_aggclick_session . ' (' .
					'session_hash		VARCHAR(200)			NOT NULL, ' .
					'UNIQUE INDEX(session_hash)) ' . $wgDBTableOptions;
			$db->query($sql, $fname);
			$count++;
		}
		
		return WoogleUtil::profileOut(__METHOD__, $count);
	}
	
	/**
	 * Normalizes a keyword query by
	 * - trimming surrounding spaces
	 * - lowercasing
	 * 
	 * @param $query			String, keyword query
	 * @return String			normalized keyword query
	 */
	public static function normalizeQueryString($query){
		WoogleUtil::profileIn(__METHOD__);
		$res = trim($query);
		$res = mb_strtolower($res);
		WoogleUtil::profileOut(__METHOD__);
		return $res;
	}
	
	public static function getHash($searchString) {
		WoogleUtil::profileIn(__METHOD__);
		return WoogleUtil::profileOut(__METHOD__, md5($searchString));
	}
	
}

?>
