<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2009 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik and der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

set_include_path(get_include_path() . PATH_SEPARATOR . dirname(__FILE__));
require_once 'Zend/Search/Lucene.php';

/**
 * This is the PHP native implementation of WoogleQueryService, retrieving results
 * from an index maintained by Zend_Lucene_Search
 */
class WoogleQueryServiceNative extends WoogleQueryService {
	
	public function checkQueryService(){
		WoogleUtil::profileIn(__METHOD__);
		return WoogleUtil::profileOut(__METHOD__, WoogleUtil::getIndexService()->numDocs());
	}
	
	public function isNative(){
		return true;
	}
	
	
	public function doAddClick($searchString, $url, $position, $session) {
		WoogleUtil::profileIn(__METHOD__);
		WoogleUtil::debug('Adding click for URL: ' . $url . ' / query: ' . $searchString);
		WoogleUtil::getQueryManager()->handleClick($searchString, $url, $position, $session);
		WoogleUtil::profileIn(__METHOD__);
	}
	
	public function getSortKeys() {
		WoogleUtil::profileIn(__METHOD__);
		return WoogleUtil::profileOut(__METHOD__, array(	WoogleUtil::getMsg('OrderByRelevance'),
															WoogleUtil::getMsg('OrderByID'),
															WoogleUtil::getMsg('OrderByLabel'),
															WoogleUtil::getMsg('OrderByMimetype'),
															WoogleUtil::getMsg('OrderByChanged'),
															WoogleUtil::getMsg('OrderBySize')
															)
										);

	}

	public function getAutocomplete($autocomplete, $num) {

		WoogleUtil::profileIn(__METHOD__);
		$qStart = microtime(true);
		
		WoogleUtil::debug( 'Calling autocomplete with: ' . $autocomplete .' and num: '.$num);

//		$autocomplete = utf8_decode($autocomplete);		// this is probably required due to MediaWiki
//		WoogleUtil::debug( 'Calling autocomplete with (decoded): ' . $autocomplete);
		
		//Zend_Search_Lucene_Search_QueryParser::setDefaultEncoding('utf-8');
		
		//Zend_Search_Lucene_Analysis_Analyzer::setDefault(new Zend_Search_Lucene_Analysis_Analyzer_Common_Utf8_CaseInsensitive() );
		Zend_Search_Lucene_Analysis_Analyzer::setDefault( WoogleLucene::getAnalyzer() );
		
		$index = WoogleLucene::getIndex();
		$q = new Zend_Search_Lucene_Search_Query_Boolean();

		// query content
		if ( ! WoogleConfig::get('restrictAutoComplete') ) {
			//$pContent = new Zend_Search_Lucene_Index_Term($autocomplete . '*', 'content');
			//$qContent = new Zend_Search_Lucene_Search_Query_Wildcard($pContent);
			$qContent = Zend_Search_Lucene_Search_QueryParser::parse(WoogleFd::$content . ':' . $autocomplete . '*' , 'utf-8');
			$q->addSubQuery($qContent, null);
		}

		// query labels
		//$pLabel = new Zend_Search_Lucene_Index_Term($autocomplete . '*', 'label');
		//$qLabel = new Zend_Search_Lucene_Search_Query_Wildcard($pLabel);
		$qLabel = Zend_Search_Lucene_Search_QueryParser::parse(WoogleFd::$label . ':' . $autocomplete . '*' ,  'utf-8');
		$q->addSubQuery($qLabel, null);

		$hits = $index->find($q);
		$result = array();
		$dups = array();

		WoogleUtil::debug( print_r($q->getQueryTerms(), true));		
		// also works for $qContent->getQueryTerms()
		
		/* yields
		 
		 Array(
	    	[0] => Zend_Search_Lucene_Index_Term Object
	        	(
	            	[field] => label
	            	[text] => veraenderung
	        	)
		)
		 
		 */
		
		// remove duplicates
		foreach ($q->getQueryTerms() as $term) {
			if (empty($dups[$term->text]) || $dups[$term->text] < $index->docFreq($term)) {
				$dups[$term->text] = $index->docFreq($term);
			}
		}
		
		// sort by frequency
		foreach ($dups as $term => $freq) {
			$prevCount = count($result);
			for ($i = 0; $i < $prevCount; $i++) {
				if ($result[$i]['freq'] < $freq || ($result[$i]['freq'] == $freq && $result[$i]['entity'] > $term)) {
					break;
				}
			}
			for ($j = $prevCount; $j > $i; $j--) {
				$result[$j] = $result[$j - 1];
			}
			$result[$i] = array('entity' => $term, 'freq' => $freq, 'name' => $term);
		}
		
		if (count($result) > $num) {
			$result = array_slice($result, 0, $num);
		}
		
		// do some logging
		$qStop = microtime(true);
		$qTime = $qStop - $qStart;
		WoogleUtil::perflog($qTime . ';' . 'autocomplete', 'query');
		WoogleUtil::profileOut(__METHOD__);
		return $result;
	}

	public function getAttributeKeys() {
		WoogleUtil::profileIn(__METHOD__);
		return WoogleUtil::profileOut(__METHOD__, array(WoogleFd::$mimetype));
	}
	
	// TODO: this seems to be a *very* expensive method on large indices
	public function getAttributeValues($key) {
		WoogleUtil::profileIn(__METHOD__);
		
		if ($key == WoogleFd::$mimetype) {
			$mimetypeFile = WoogleLucene::getIndexPath() . '/mimetypes.dat';
			if (!file_exists($mimetypeFile) || ((time() - filemtime($mimetypeFile)) > 3600)) {
				$index = WoogleLucene::getIndex();
				$values = array();
				foreach ($index->terms() as $term) {
					if ($term->field == WoogleFd::$mimetype) {
						$values[] = $term->text . ';' . $index->docFreq($term);
					}
				}
				$file = fopen($mimetypeFile, 'w');
				foreach ($values as $v) {
					fwrite($file, $v . "\n");
				}
				fclose($file);
			}
			$resultValues = array();
			$fileValues = file($mimetypeFile, FILE_IGNORE_NEW_LINES);
			foreach ($fileValues as $fv) {
				$parts = explode(';', $fv);
				$resultValues[] = array('entity' => $parts[0], 'name' => $parts[0], 'freq' => $parts[1]);
			}
			WoogleUtil::profileOut(__METHOD__);
			return $resultValues;
		}
		else {
			WoogleUtil::profileOut(__METHOD__);
			return array();
		}
	}

	public function executeKeywordQueryValues($searchString, $count, $start, $values, $referer='') {
		global $wgRequest;
		WoogleUtil::profileIn(__METHOD__);
		
		//$searchString = utf8_decode($searchString);
//		$usearchString = utf8_decode($searchString);		// this is probably required due to MediaWiki
		$usearchString = $searchString;
		
		WoogleUtil::debug('sContent1: ' . $searchString);
		WoogleUtil::debug('sContent2: ' . utf8_encode($searchString));
		WoogleUtil::debug('sContent3: ' . utf8_decode($searchString));
		
		//
		// Query the Zend_Lucene_Search Index
		//
		
		$index = WoogleLucene::getIndex();
		//Zend_Search_Lucene_Search_QueryParser::setDefaultEncoding('utf-8');
		//Zend_Search_Lucene_Analysis_Analyzer::setDefault(new Zend_Search_Lucene_Analysis_Analyzer_Common_Utf8() );
		//Zend_Search_Lucene_Analysis_Analyzer::setDefault(new Zend_Search_Lucene_Analysis_Analyzer_Common_Utf8_CaseInsensitive() );
		Zend_Search_Lucene_Analysis_Analyzer::setDefault( WoogleLucene::getAnalyzer() );
		Zend_Search_Lucene_Search_QueryParser::setDefaultOperator(Zend_Search_Lucene_Search_QueryParser::B_AND); 
		
		//
		// Construct boolean query to search in content and label
		//
		Zend_Search_Lucene::setDefaultSearchField(WoogleFd::$content);
		$qContent = Zend_Search_Lucene_Search_QueryParser::parse($this->escape($usearchString), 'utf-8');
		Zend_Search_Lucene::setDefaultSearchField(WoogleFd::$label);
		$qLabel = Zend_Search_Lucene_Search_QueryParser::parse($this->escape($usearchString), 'utf-8');
		$qLabel->setBoost(2);
		$qBase = new Zend_Search_Lucene_Search_Query_Boolean();
		$qBase->addSubQuery($qContent, null);
		$qBase->addSubQuery($qLabel, null);
		$q = new Zend_Search_Lucene_Search_Query_Boolean();
		$q->addSubQuery($qBase, true);
		
		//
		// add query constraints for advanced search
		//
		
		// mimetypes
		if (count($values['mimetypes']) > 0) {
			$qMime = new Zend_Search_Lucene_Search_Query_Boolean();
			foreach ($values['mimetypes'] as $m) {
				$term  = new Zend_Search_Lucene_Index_Term($m, WoogleFd::$mimetype);
				$subQuery = new Zend_Search_Lucene_Search_Query_Term($term);
				$qMime->addSubQuery($subQuery, null);
			}
			$q->addSubQuery($qMime, true);
		}
		
		// change date
		if ($values['changedMin'] > 0 || $values['changedMax'] > 0) {
			$date = array();
			$date['changedMin'] = '00000000000000000';
			$date['changedMax'] = '99999999999999999';
			foreach ($date as $dKey => $dValue) {
				if ($values[$dKey] > 0) {
					$date[$dKey] = strftime('%Y%m%d%H%M%S', $values[$dKey]) . '000';
				}
			}
			$qChangedFrom = new Zend_Search_Lucene_Index_Term($date['changedMin'], WoogleFd::$changed);
			$qChangedTo   = new Zend_Search_Lucene_Index_Term($date['changedMax'], WoogleFd::$changed);
			$qChanged = new Zend_Search_Lucene_Search_Query_Range($qChangedFrom, $qChangedTo, true);
			$q->addSubQuery($qChanged, true);
		}
		
		// file size
		if ($values['sizeMin'] > 0 || $values['sizeMax'] > 0) {
			$size = array();
			$size['sizeMin'] = 0;
			$size['sizeMax'] = PHP_INT_MAX;
			foreach ($size as $sKey => $sValue) {
				if ($values[$sKey] > 0) {
					$size[$sKey] = WoogleUtil::longToString($values[$sKey]);
				}
			}
			$qSizeFrom = new Zend_Search_Lucene_Index_Term($size['sizeMin'], WoogleFd::$size);
			$qSizeTo   = new Zend_Search_Lucene_Index_Term($size['sizeMax'], WoogleFd::$size);
			$qSize = new Zend_Search_Lucene_Search_Query_Range($qSizeFrom, $qSizeTo, true);
			$q->addSubQuery($qSize, true);
		}
		
		// exclude Woogle page representing the actual query (=query itself) from result set
		if ($values['excludeId'] > 0) {
			$id = WoogleQueryServiceCommon::getId($values['excludeId']);
			$term = new Zend_Search_Lucene_Index_Term($id, WoogleFd::$id);
			$subQuery = new Zend_Search_Lucene_Search_Query_Term($term);
			$q->addSubQuery($subQuery, false);
		}
		
		//
		//	Get hits from index
		//
		$hits;
		if ($values['sortType'] > 0) {
			$hits = $index->find($q, $this->getSortField($values['sortType']), SORT_REGULAR, $values['sortDesc'] ? SORT_DESC : SORT_ASC);
		} else {
			$hits = $index->find($q);
		}
		
		//
		//	Build result set from hits
		//
		// $results = new WoogleResultList(count($hits));		
		//XXX oldcode
		$result = array('numHits' => count($hits));
		
		// iterate requested number (start, count) of hits
		for ($i = $start; ($i < ($count + $start)) && ($i < count($hits)); $i++) {
			$doc = $hits[$i]->getDocument();
			
			$result['results'][$i - $start]['relevance'] = $hits[$i]->score;
			
			// iterate document fields
			foreach ($doc->getFieldNames() as $docField) {

				//$val = urldecode(utf8_encode($doc->getFieldValue($docField)));
				$val = $doc->getFieldValue($docField);
				
				if ($docField == WoogleFd::$content) {
					
					/*
					// remove wiki-text
					// this too expensive at this place!  
					  
					global $wgParser, $wgUser;
					$po = ParserOptions::newFromUser($wgUser);
					$po->setIsPrintable(true);
					$pOut = $wgParser->parse($val, Title::newFromText($doc->getFieldValue(WoogleFd::$label)) , $po);
					$val = $pOut->getText();
					*/
					
					// assume HTML or wikitext
					$val = strip_tags($val);

					// TODO Highlighting
					//self::highlight3($val, $q->getQueryTerms());
					
					// make array of strings
					$hTerms = array();
					foreach ($qContent->getQueryTerms() as $hTerm) {
						array_push($hTerms, $hTerm->text);
					}
					
					$val = self::highlight3($val, $hTerms);
		//			$hquery = Zend_Search_Lucene_Search_QueryParser::parse($searchString, 'UTF-8', new WoogleZendSearchHighlighter());
		//			$highlightedHTML = $query->highlightMatches($val);
		//			$val = $highlightedHTML;

					// TODO Cut snippet size
					if (strlen($val) > 0) $val = self::cut($val, 200) . '</b></b>'; // TODO bad hack ;-)
			/*
					// cut snippet size
					$firstpos = stripos($val, $searchString);
					$snippet_from = max(0, $firstpos - 20);
					$snippet_to = $firstpos - $snippet_from + strlen($searchString) + 150;
					$val = "... " . substr($val, $snippet_from, $snippet_to) . " ...";
					
					// Debug
					WoogleUtil::debug('Content1: ' . $val);
					WoogleUtil::debug('Content2: ' . utf8_encode($val));
					WoogleUtil::debug('Content3: ' . utf8_decode($val));
				
					// highlight query matches
					$val = self::highlight2($val, $searchString);
			*/
				
				} elseif ($docField == WoogleFd::$size) {
					$val = WoogleUtil::stringToLong($val);
				} elseif ($docField == WoogleFd::$sizeWords) {
					$val = WoogleUtil::stringToLong($val);
				} elseif ($docField == WoogleFd::$sizeLines) {
					$val = WoogleUtil::stringToLong($val);
				} elseif ($docField == WoogleFd::$sizeChars) {
					$val = WoogleUtil::stringToLong($val);
				} elseif ($docField == WoogleFd::$label){
					// highlight query matches
					//$val = self::highlight3($val, $qLabel->getQueryTerms()); TODO enable again (causes problems with wiki page detection in WoogleSearch)
					//$val = self::highlight($val, $searchString);
				} elseif ($docField == WoogleFd::$created){
					// 20100302194122000
					$val = substr($val, 0, 14);
				} elseif ($docField == WoogleFd::$changed){
					$val = substr($val, 0, 14); 
				}
			
				$result['results'][$i - $start][$docField] = $val;				

				/* zend lucene highlighting (http://framework.zend.com/manual/en/zend.search.lucene.searching.html#zend.search.lucene.searching.highlighting)
				if ($docField == 'content' || $docField == 'label') {
					$result['results'][$i - $start][$docField] = $q->highlightMatches($result['results'][$i - $start][$docField]);
				}*/
				
			}
			
		}
		
		//
		//		Log query
		//
		
		// TODO rendundant w.r.t. $start ?
		$page = $wgRequest->getInt('page', -1);
		$browse = true;
		if ($page < 1) {
			$browse = false;
		}
		$sessionHash = $wgRequest->getText('session', md5(mt_rand())); // 2nd parameter = default value
		
		if ($browse) {
			// additional click to a given query ("browsing results")
			WoogleUtil::getQueryManager()->handleBrowse($searchString, $sessionHash);
		} else {
			// new query - first result page
			WoogleUtil::getQueryManager()->handleQuery($searchString, $result['numHits']);
		}
		/*
		// old query logging into db
		$db = WoogleMWUtil::getDB();
		extract($db->tableNames('woogle_query'));
		$sql = 'INSERT INTO ' . $woogle_query . ' SET ' .
				'query_time		= ' . time() . ', ' .
				'query_referer	= "' . mysql_real_escape_string($referer) . '", ' .
				'query_remoteip	= "' . mysql_real_escape_string(getenv('REMOTE_ADDR')) . '", ' .
				'query_user		= ' . mysql_real_escape_string($wgUser->getID()) . ', ' .
				'query_query	= "' . mysql_real_escape_string($this->searchString) . '", ' .
				'query_numhits	= ' . mysql_real_escape_string(intval($resultList['numHits'])) . ', ' .
				'query_count	= ' . mysql_real_escape_string($resultCount) . ', ' .
				'query_start	= ' . mysql_real_escape_string($resultStart);
		$db->query($sql);
		$db->commit();
		*/
		
		WoogleUtil::profileOut(__METHOD__);
		return $result;
	}
	

	public function supportPages() {
		WoogleUtil::profileIn(__METHOD__);
		// TODO
		return WoogleUtil::profileOut(__METHOD__, false);
	}

	
	private function getSortField($num) {
		WoogleUtil::profileIn(__METHOD__);
		switch ($num) {
			case 0:	return WoogleUtil::profileOut(__METHOD__, '');
			case 1:	return WoogleUtil::profileOut(__METHOD__, WoogleFd::$id);
			case 2: return WoogleUtil::profileOut(__METHOD__, WoogleFd::$label);
			case 3:	return WoogleUtil::profileOut(__METHOD__, WoogleFd::$mimetype);
			case 4:	return WoogleUtil::profileOut(__METHOD__, WoogleFd::$changed);
			case 5:	return WoogleUtil::profileOut(__METHOD__, WoogleFd::$size);
		}
	}
	
	/**
	 * TODO Doc?
	 * 
	 * @param $string			string, with characters to escape
	 * @return 					string, with escaped characters
	 */
	public static function escape($string) {
		WoogleUtil::profileIn(__METHOD__);
		//$chars = array('+', '-', '&', '|', '!', '(', ')', '{', '}', '[', ']', '^', '"', '~', '*', '?', ':', '\\');
		// +, - etc. shall be allowed
		$chars = array('{', '}', '[', ']', '~', ':', '\\');
		foreach ($chars as $c) {
			$string = str_replace($c, '\\' . $c, $string);
		}
		WoogleUtil::profileOut(__METHOD__);
		return $string;
	}
	
	/**
	 * 
	 * @param $text
	 * @param $to_highlight
	 * @return String
	 */
	public static function highlight2($text, $to_highlight){
		
		$tx = mb_strlen($to_highlight);
		$x1 = '<b style="color:black;background-color:#ff66ff">';
		$xx = mb_strlen($x1); 
		$x2 = '</b>';
		
		$len 	= mb_strlen($text);
		$match 	= mb_stripos($text, $to_highlight);
		$text 	= mb_substr($text, 0, $match) . $x1 . $to_highlight . $x2 . mb_substr($text, $match + $tx, $len);
		//$text = substr($text, 0, $match) . $x1 . substr($text, $match, $match+$tx) . $x2 . substr($text, $match + $tx, $len);  
		
		return $text;
	}

	
/*
	// important: method should not scan full document unless necessary
	
	public static function makeSnippets($text, $terms, $fragmentsize=180, $snippetnum=3, $highlight=true){
		
		// TODO vs. param?
		$snippetsize = $fragmentsize/$snippetnum;

		$matches = array();
		$s = 0;
		$found = true;

		// collect matching snippets util n fragments do not overlap
		while ((s <= $snippetnum) && $found ){
			
			// iterate all terms
			foreach ($terms as $term){
				$p = mb_stripos($text, $term);
				 //mb_stripos  ( string $haystack  , string $needle  [, int $offset  [, string $encoding  ]] )
				 
			}
			
		}
		
		
	}
*/
	
	
	/**
	 * 
	 * @param $haystack			String, long text
	 * @param $needle			array of Strings to search in long text
	 * @return 					String, long text with needle-terms highlighted
	 */
	public static function highlight3($haystack, $needle){

		/* for wilcard queries, $needle will contain subsuming queries (e.g. "animal*" might
		   return "aninmal" as well as "animals" 
		
		   in order to mark any match, and to avoid double highlighting one needs to start
		   with the most specific terms
		
		*/
		
		// iterate all terms
		foreach ($needle as $term) {
			
			// iterate all occurences of term in text
			$i = mb_stripos($haystack, $term);
			
			//echo 'Initial i for ' . $term->text . ' is: ' . $i . '<br\>\n\n<br \>';

			// TODO - this behaves strange for large volumes of data with no content matches
			// $n is a temporary fix only
			$n = 0;
			while (($i > -1) && ($n < 100)) {
				$n++;
				$l = mb_strlen($term);
				$haystack = mb_substr($haystack, 0, $i) . WoogleQueryService::$highlightStart . 
							mb_substr($haystack, $i, $l) . WoogleQueryService::$highlightStop .
							mb_substr($haystack, $i+$l, mb_strlen($haystack));
				
				$i = mb_stripos($haystack, $term, $i+ mb_strlen(WoogleQueryService::$highlightStart)+1);
				//echo 'Further i for ' . $term->text . ' is: ' . $i . '<br\>\n\n<br \>';
			}	
		}
		
		//echo 'QueryTerms: ' . print_r($needle, true) . '<br\>\n\n<br \>';
		//echo 'Haystack: ' . $haystack . '<br\>\n\n<br \>';
		
		return $haystack;
	}
	
	/**
	 * 
	 * @param $haystack			string, long text
	 * @param $size				int, maximum size $haystack should be cut to
	 * @return 					string, $haystack reduced to $size
	 */
	public static function cut($haystack, $size){
		$result = $haystack;
		// do simple implementation first - probably later merged with highlight3...
		
		// only cut large stuff
		if (mb_strlen($haystack) > $size){
			$i = mb_stripos($haystack, WoogleQueryService::$highlightStart);
			
			if ($i > -1){
				
				$maxstart = mb_strlen($haystack) - $size; // maximum start point if full size should be achieved
				$minstart = max(0, $i-30); // begin $i-30 but at least 0
				$start = min($minstart, $maxstart); // $start must always allow $start+$size != strlen($haystack)
				
				$result = mb_substr($haystack, $start, $size);
				if ($start > 0) $result = "..." . $result;
				if (($start+$size) < mb_strlen($haystack)) $result = $result . "...";
				
				//echo 'CUT FROM ' . $start . ' TO ' . ($start+$size);
				
			} else {
				// no highlight - plain cut
				$result = mb_substr($haystack, 0, $size); 
			}
			
		}
		
		return $result;
	}
	
	
	
}

?>
