<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2009 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik and der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

/**
 * Creates/drops database tables for Woogle. This is equivalent to clicking the respective
 * button on the page Special:WoogleConfig.
 *
 * Note: if Woogle is not installed in its standard path under ./extensions
 *       then the MW_INSTALL_PATH environment variable must be set.
 *       See README in the maintenance directory.
 *
 * Usage: see printUsage() below
 */


/*
 * no guarantees, but look in the usual place for commandLine.inc, so this
 * so it will work most of the time
 */
require_once ( getenv('MW_INSTALL_PATH') !== false
    ? getenv('MW_INSTALL_PATH')."/maintenance/commandLine.inc"
    : dirname( __FILE__ ) . '/../../../../../maintenance/commandLine.inc' );
require_once("$IP/maintenance/counter.php");


/* user/password in LocalSettings probably don't have the rights we need,
 * so allow override
 * Note: the preferred method is to use AdminSettings.php to provide such credentials
 */
if( isset( $options['user'] ) ) {
	global $wgDBuser;
	$wgDBuser = $options['user'];
}
if( isset( $options['password'] ) ) {
	global $wgDBuser;
	$wgDBpassword = $options['password'];
}

require_once('../WoogleQueryManagerNative.php');

// delete tables
if (  array_key_exists( 'delete', $options ) ) {
	deleteWoogleTables();
// create tables
} else if (  array_key_exists( 'create', $options ) ){
	createWoogleTables();
} else {
	printUsage();
}

function printUsage(){
	echo '
 * USAGE INSTRUCTIONS
 * =======================================================
 * php WoogleNativeDb_setup.php [options...]
 *
 * --create			Create Woogle database tables.
 * --delete			Delete Woogle database tables.
 * --user     <dbuser>      	Database user account to use for changing DB layout. If not set,
 * 				the credentials in AdminSettings.php are used.
 * --password <dbpassword>  	Password for user account to use. (Instead of custom password.)
 * 
 * NOTE: specifying user credentials in a command line call will usually store them
 * within the shell history file. For security, provide credentials in AdminSettings.php
 * instead and ensure that your text editor does not create world-readable backup copies
 * when modifying this file.
 ';
}

function createWoogleTables(){
	$count = 0;
	
	// check if tables not exist
	if (!WoogleMWUtil::tableExists('woogle_aggquery')){
		try {
			$count = WoogleQueryManagerNative::initDb();
			echo "Done! (Created " . $count . ' tables in the database.)';
		}
		catch (Exception $e) {
			echo 'Error! (Database error - check your access rights: ' . $e . ')';
		}
	} else {
		echo 'Error! (Table woogle_aggquery already exists!)';
	}
}

function deleteWoogleTables(){
	// check if tables exist
	if (WoogleMWUtil::tableExists('woogle_aggquery')){
		
		print "WARNING: Deleting all Woogle tables!\n\n";
		$delay=10;
		print "Abort with CTRL-C in the next $delay seconds ...   ";
	
		for ($i = $delay+1; $i >= 1;) {
			echo str_repeat( chr(8), strlen( $i ) ) . str_repeat( chr(255), strlen( $i ) ) .
		    	 str_repeat( chr(8), strlen( $i ) ) . --$i;
			sleep(1);
		}
	
		$count = 0;
		$count = WoogleQueryManagerNative::dropDb();
	
		while (ob_get_level() > 0) { // be sure to have some buffer, otherwise some PHPs complain
			ob_end_flush();
		}
	
		echo "Done! ($count tables have been deleted. You can recreate them with this script.)";
	} else {
		echo 'Error! (Table woogle_aggquery does not exist!)';
	}
}



?>