<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2009 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik and der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

interface WoogleQueryManager {
	public function handleQuery($searchString, $numHits);
	
	public function handleBrowse($searchString, $sessionHash);
	
	public function handleClick($searchString, $url, $position, $session);
	
	/**
	 * @param $number		int, number of queries to return
	 * @return				array(
	 * 							array(
	 * 								'queryId'		=> string,
	 * 								'localNDFUsers'	=> array(string, ...),
	 * 								'queryUsers'	=> array(string, ...),
	 * 							),
	 * 							...
	 * 						)
	 */
	public function getUnsatisfiedQueriesWithFewHits($number);
	
	/**
	 * @see getUnsatisfiedQueriesWithFewHits($number)
	 */
	public function getUnsatisfiedQueriesWithManyDifferentClicks($number);
	
	/**
	 * @see getUnsatisfiedQueriesWithFewHits($number)
	 */
	public function getUnsatisfiedQueriesWithMuchBrowsing($number);
	
	/**
	 * @param $number		int, number of queries to return
	 * @return				array(
	 * 							@see getUnsatisfiedQueriesAggregations($number),
	 * 							...
	 * 						)
	 */
	public function getUnsatisfiedQueriesAggregations($number);
	
	/**
	 * @param $query		string, which query to get the information need for
	 * @return				array(
	 * 							'needString'									=> string,
	 * 							'needExecutions'								=> int,
	 * 							'needExecutionsWithoutResultClick'				=> int,
	 * 							'recency'										=> timestamp (int),
	 * 							'resultHits'									=> int,
	 * 							'firstDate'										=> timestamp (int),
	 * 							'lastDate'										=> timestamp (int),
	 * 							'sizeQueryUsers'								=> int,
	 * 							'publicQueryUsers'								=> array(string, ...),
	 * 							'needExecutionsWithFurtherPages'				=> int,
	 * 							'needExecutionsWithFurtherPagesAveragePages'	=> float,
	 * 							'resultClicks'									=> int,
	 * 							'uniqueClickedResults'							=> int,
	 * 						)
	 */
	public function getInformationNeed($searchString);
	
	/**
	 * @param $query		string, which query to get related queries for
	 * @param $max			int, maximum number of queries to be returned
	 * @param $excludeQuery	bool, if given query should be excluded from the result
	 * @return				@see getUnsatisfiedQueriesWithFewHits($number)
	 */
	public function getRelatedQueries($searchString, $max, $excludeQuery);
	
	/**
	 * @param $number		int, number of queries to return
	 * @return				WoogleTermFreq
	 */
	public function getPopularQueries($number);
	
	/**
	 * @param $number		int, number of queries to return
	 * @return				WoogleEntityFreq
	 */
	public function getPopularResults($number);
	
	/**
	 * @param $number		int, number of queries to return
	 * @return				WoogleEntityFreq
	 */
	public function getPopularSources($number);
	
	/**
	 * Returns term/frequency pairs for queries preceding a search result click
	 * 
	 * @param $uri			String, uri of a clickable search result
	 * @return				WoogleTermFreq
	 */
	public function getTags($uri);
	
	public function getAvgPages();
	
	public function getAvgRecency();
}
?>
