<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2009 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik and der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

/**
 *	This class renders the Woogle search form and result list
 *
 *	It is typically called from either
 *  @see WoogleSpecialSearch.php (= Special:Woogle)
 *  @see WoogleArticleSearch.php (= Woogle:Query)
 *  
 *  It is indirectly called via @see WoogleSpecialDummySearch.php
 *  (= replacing Special:Search)
 *  
 *  TODO: refactor
 * 
 */
class WoogleSearch {
	private $searchString;
	private $article;
	private $error = array();
	private $advancedCall = false;
	
	/**	 
	 * 
	 * @param $advancedCall		boolean (called true by WoogleArticleSearch.php, false by WoogleSpecialSearch)
	 * @return void
	 */
	public function __construct($advancedCall) {
		WoogleUtil::profileIn(__METHOD__);
		$this->advancedCall = $advancedCall;
		WoogleUtil::profileOut(__METHOD__);	 
	}
	
	/**
	 * This value is set in @see WoogleArticleSearch.php
	 * 
	 * @param 	$article	MediaWiki article object for the Woogle:Page called
	 * @return 	void
	 */
	public function setArticle($article) {
		WoogleUtil::profileIn(__METHOD__);
		$this->article 		= $article;
		//$this->advancedCall = true;
		WoogleUtil::profileOut(__METHOD__);
	}
	
	/**
	 * This value is set calling from @see WoogleSpecialSearch
	 * 
	 * @param $searchString 	string, query
	 * @return void
	 */
	public function setSearchString($searchString) {
		WoogleUtil::profileIn(__METHOD__);
		$this->searchString = $searchString;
		//$this->advancedCall = false;
		WoogleUtil::profileOut(__METHOD__);
	}
	
	public function getSearchString() {
		WoogleUtil::profileIn(__METHOD__);
		return WoogleUtil::profileOut(__METHOD__, $this->searchString);
	}
	


	/**
	 * Main method of this class which renders the search form and the result list
	 * 
	 * @return 	void
	 */
	public function show() {
		global $wgRequest, $wgOut, $wgUser, $wgScriptPath, $wgUseAjax, $wgArticlePath, $wgUsePathInfo;
		WoogleUtil::profileIn(__METHOD__);
		
		$startTime = microtime(true);

		//
		// 	Check if user is allowed to use Woogle
		//
		if (!WoogleConfig::hasPermission() || !WoogleConfig::isEnabled()) {
			$wgOut->setPageTitle(wfMsg('woogleNoPermissionTitle'));
			//$this->error[] = wfMsg('woogleNoPermission'); // $par, SpecialPage::getTitleFor('Search')->getPrefixedText()
			$this->error[] = WoogleUtil::getMsg('NoPermission', SpecialPage::getTitleFor('Search')->getPrefixedText()); // TODO
			$this->errorMsg();
			$this->helpMsg();
			$this->showBuiltInSearch();
			WoogleUtil::debug('WoogleSearch: Exit because user has no permission');
			WoogleUtil::profileOut(__METHOD__);
			return;
		}
		
		
		/*
		 * 	/wiki/index.php/Woogle:Test
		 * 	/wiki/index.php/Woogle:Test?advanced=true&repositories[]=33
		 *	/wiki/index.php/Special:Woogle/Test
		 *	or:	
		 *	/wiki/index.php?title=Special:Woogle/Test
		 *	/wiki/index.php?title=Special:Woogle/Test&p1=v1&p2=v2
		 */
		$requestUrl		= $wgRequest->getRequestURL();
		
		/*
		 * 	/wiki/index.php/Woogle:Test
		 * 	/wiki/index.php/Woogle:Test, advanced=true&repositories[]=33
		 *	/wiki/index.php/Special:Woogle/Test
		 */	
		
		// if we have index.php?title=... mode
		if ($wgUsePathInfo === false) {
			$requestUrlExploded = explode('&', $requestUrl, 2);
		} else {
			$requestUrlExploded = explode('?', $requestUrl);
		}

		/*
		 (  [0] => 
		    [1] => wiki
		    [2] => index.php
		    [3] => Woogle:Test	)
		
		(   [0] => 
		    [1] => wiki
		    [2] => index.php
		    [3] => Special:Woogle
		    [4] => Test		)
		 */
		//$requestParts 	= explode('/', $requestUrl[0]); //unneeded
		
		/*
		 * empty or "@advanced=true&repositories[]=33" with @ = & or ? (depending on wgUsePathInfo)
		 */
		if (!empty($requestUrlExploded[1])){
			$trail 			= WoogleUtil::extractField($requestUrlExploded[1], 'searchString');
			// if we have index.php?title=... mode
			if ($wgUsePathInfo === false) {
				$trail	= WoogleUtil::extractField($trail, 'title');
				$trail  = !empty($trail) ? '&' . $trail : '';
			} else {
				$trail 	= !empty($trail) ? '?' . $trail : '';
			}
		}
	
		
		//
		//		BLOCK STARTS
		//
		// TODO: Check if this can be a problem in the index.php?title=... mode, too.
		if ($requestUrlExploded[0] == (WoogleMWUtil::getArticlePath() . WoogleUtil::getWoogleSearchSpecialPageName())) {
			// 	missing trailing slash: ("/wiki/index.php/Special:Woogle")
			$wgOut->redirect(WoogleUtil::getWoogleLinkPrefixNonadvanced() . $trail);
			// redirect to e.g. "/wiki/index.php/Special:Woogle/" . "?searchString=Test" 
			WoogleUtil::debug('WoogleSearch: Redirect somehow');
			WoogleUtil::profileOut(__METHOD__);
			return;
			
		} 
		/*
		elseif ($requestUrlExploded[0] == WoogleUtil::getWoogleLinkPrefixNonadvanced()) {
			// with trailing slash: ("/wiki/index.php/Special:Woogle/")
			WoogleUtil::debug('WoogleSearch: Setting searchString empty');
			$this->searchString = '';
		}*/ 
		
		if (stristr(urldecode($requestUrlExploded[0]), '#')) {
			// somehow MW cannot handle articles with a # in it, even if it's urlencoded
			$this->searchString = '';
			// time for a little easter egg :-)
			$this->error[] = 'Including # in your search makes baby jesus cry.';
		}

		if ($this->advancedCall) {
			// called in advanced mode
			$sstring = urldecode($this->article->getTitle()->getText());
			//$this->searchString = strtolower($sstring);	// normalize Woogle pages
			$this->searchString = mb_strtolower($sstring);	// normalize Woogle pages
			WoogleUtil::debug('WoogleSearch: Advanced call - decoding ' . $this->article->getTitle()->getText() . ' to ' . $this->searchString);
			// are we in non-advanced mode?
			if (!WoogleConfig::get('advanced')) {
				WoogleUtil::debug('WoogleSearch: Redirect to nonadvanced');
				$wgOut->redirect(WoogleUtil::getWoogleLinkPrefixNonadvanced() . $this->searchString . $trail);
				WoogleUtil::profileOut(__METHOD__);
				return;
			}
		} else {
			// called in non-advanced mode
			
			// searchString is already set by WoogleSpecialSearch
			WoogleUtil::debug('WoogleSearch: Non-advanced call - decoding ' . $this->searchString . ' to ' . urldecode($this->searchString));
			$this->searchString = urldecode($this->searchString);
			// remove underscores
			$this->searchString = str_replace('_', ' ', $this->searchString);
			// are we in advanced mode?
			if (WoogleConfig::get('advanced') && !empty($this->searchString)) {
				WoogleUtil::debug('WoogleSearch: Redirect to advanced');
				$wgOut->redirect(WoogleUtil::getWoogleLinkPrefixAdvanced() . $this->searchString . $trail);
				WoogleUtil::profileOut(__METHOD__);
				return;
			}
		}

		//
		//		BLOCK ENDS
		//
		
		
		//
		//	Check if "go" feature is used (index.php?title=Special%3ASearch&search=test&go=Go)
		//
		$togo = $wgRequest->getVal('go');
		if (isset($togo)){
			WoogleUtil::debug('Go-parameter is set to: ' . $togo);
			
			$t_go 		= new Title();
			$togo_page 	= $this->searchString;
			$tt_go 		= $t_go->newFromDBkey($togo_page);
			
			// remove "Special" from potential Special:Page string
			$spec = substr( $this->searchString, strpos($this->searchString, ':')+1 , strlen($this->searchString));
			
			// check if there is an article or a special page matching the value of the "go" parameter
			if ((isset($tt_go) && $tt_go->exists()) || SpecialPage::exists($spec)) {
				WoogleUtil::debug('Article ' . $togo_page . ' exists - redirecting');
				// log query before exiting
				wfRunHooks('WoogleSearchLogQuery', array($togo_page . ';GO!;' . ';REF:;' . $referer));
				// redirect to matching article
				$wgOut->redirect(str_replace('$1', $togo_page, $wgArticlePath));
				WoogleUtil::profileOut(__METHOD__);
				return;
			} else {
				WoogleUtil::debug('Article ' . $togo_page . ' does not exist - executing search');
			}
		}
		

		// run hook
		// checks if user has to confirm evaluation study participation (among possible other)
		// TODO - this should happen before redirect
		if (!wfRunHooks('WoogleSearchInit', array($this))) {
			WoogleUtil::debug('WoogleSearch: Exit because WoogleSearchInit hook returned false');
			//$this->showBuiltInSearch();
			WoogleUtil::profileOut(__METHOD__);
			return;
		}
		
		
		//
		//		'ref' parameter
		//
		
		
		// redirect if get-parameter "ref" is set
		if (!empty($_GET['ref']) && count($this->error) == 0) {
			// remove 'ref' value from request url
			$newFields = WoogleUtil::extractField($requestUrlExploded[1], 'ref');
			setcookie('ref', $_GET['ref']);
			
			if ($wgUsePathInfo === false) { // if we have index.php?title=... mode
				$wgOut->redirect(utf8_encode($requestUrlExploded[0]) . (!empty($newFields) ? '&' . $newFields : ''));
			} else {
				$wgOut->redirect(utf8_encode($requestUrlExploded[0]) . (!empty($newFields) ? '?' . $newFields : ''));
			}
			
			WoogleUtil::debug('WoogleSearch: Redirect REF');
			WoogleUtil::profileOut(__METHOD__);
			return;
		}
		
		setcookie('ref', '');
		
		$referer = '';
		if (!empty($_COOKIE['ref'])) {
			$referer = $_COOKIE['ref'];
		}
		
		
		//
		//		Advanced search preparation
		//
		WoogleUtil::debug('WoogleSearch: Advanced search preparation');
		
		// get parameters for the advanced search
		$advanced = array();
		$advancedSingle = array('changedMin', 'changedMax', 'sizeMin', 'sizeMax', 'sizeFactor', 'pagesMin', 'pagesMax', 'sortType', 'sortDesc');
		foreach ($advancedSingle as $as) {
			$advanced[$as] = $wgRequest->getText($as, '');
		}
		$dates = array('changedMin' => 'Time period from', 'changedMax' => 'Time period to');
		foreach ($dates as $date => $dateDesc) {
			$smallDate = strtotime($advanced[$date]);
			if ($smallDate == false && !empty($advanced[$date])) {
				$this->error[] = '"' . $dateDesc . '" contains an invalid date.';
			}
		}
		
		$linker = new Linker();

		$wgOut->setArticleRelated( false );
		$wgOut->setRobotPolicy( "noindex,follow" );
		
		$showCountDropdown = '<select name="count" size="1" onchange="countCookie()"><option value="' . WoogleConfig::get('defaultCount') . '">' . WoogleConfig::get('defaultCount') . '</option>';
		for ($i = WoogleConfig::get('defaultCount') * 2.5; $i <= WoogleConfig::get('defaultCount') * 20; $i *= 2) {
			$showCountDropdown .= '<option' . (($wgRequest->getInt('count') == $i) ? ' selected="selected"' : '') . '>' . $i . '</option>';
		}
		$showCountDropdown .= '</select>';
		$showCountCheckbox = '<input type="checkbox" name="countSave" value="1" style="margin: 0px;" onclick="countCookie()" />';
		$sizeFactors = array(array('', 'Byte'), array('1', 'KB'), array('2', 'MB'), array('3', 'GB'));

		
		//
		//		Advanced search form
		//
		WoogleUtil::debug('WoogleSearch: Advanced search form');
		
		// get list of sources / mimetypes from backend
		$qs = WoogleUtil::getQueryService();
		$multipleRepositories = false;
		$oAdvanced = '<div id="advancedCell"><input type="hidden" id="advanced" name="advanced" value="true" />';
		$oError = "";
		try {
			// map keys are used to store mutiple selection fields
			// these are get parameters - mimetypes[], repositories[]
			$mapKeys = array('mimetype' => 'mimetypes', 'source' => 'sources', 'repository' => 'repositories');
			$advancedKeys  = array('repository', 'mimetype'); // TODO HACK BY HJ $qs->getAttributeKeys(); // = source, mimetype
			foreach ($advancedKeys as $key) {
			
				$values = array();
				if (! $qs->isNative()) $values = $qs->getAttributeValues($key);
				
				// only investigate further, if there is more than 1 option for this key
				if (count($values) > 1) {
					if ($key == 'repository') $multipleRepositories = true;
					
					$oAdvanced .= '<div><div>' . WoogleUtil::getMsg('SearchAdvanced' . $key) . '</div>';
					$oAdvanced .= '<select name="' . $mapKeys[$key] . '[]" multiple="multiple" size="4">';
					
					// get parameters from previous call
					if ($wgRequest->getArray($mapKeys[$key]) != null) {
						if ($key == 'repository'){
							$advanced[$mapKeys[$key]] = $wgRequest->getIntArray($mapKeys[$key]);
						} else {
							$advanced[$mapKeys[$key]] = $wgRequest->getArray($mapKeys[$key]);
						}
					} else { // no values for current key
						$advanced[$mapKeys[$key]] = array();

						// 	Set default repository on wiki search or if search string is empty
						if ($key == 'repository'){
							if ( WoogleConfig::get('searchWikiOnlyAsDefault') && ($referer == 'mw' || empty($this->searchString)) ){
								$advanced['repositories'] = array(WoogleConfig::get('indexRepository'));
							}
						}
					}
					
					foreach ($values as $value) {
						$selected = false;
						foreach ($advanced[$mapKeys[$key]] as $rv) {
							if ($rv == $value['entity']) {
								$selected = true;
								break;
							} 
						}
						$oAdvanced .= '<option value="' . $value['entity'] . '"' . ($selected ? ' selected="selected"' : '') . '>' . $value['name'] . ' (' . $value['freq'] . ')</option>';
						
						/*
						// add subterms (in case of sources/repositories)
						$subTerms = $value['subterms'];
						if (isset($subterms)){
							foreach ($subterms as $subterm){
								$oAdvanced .= '<option value="' . $subterm['entity'] . '"' . ($selected ? ' selected="selected"' : '') . '>' . $subterm['name'] . ' (' . $subterm['freq'] . ')</option>';
							}
						}
						*/
					
					}
					$oAdvanced .= '</select></div>'; // TODO HJ added BR
				}
			}
		} catch (Exception $e) {
			$advancedKeys = array();
			$this->handleException($e, $this->searchString);
			$this->showBuiltInSearch();
			WoogleUtil::profileOut(__METHOD__);
			return;
		}
		
		// additional fields for advanced search (time period, file size, sort by, number of results per page
		/*
		$oAdvanced .= '
			<div style="clear:left; float:left"><div>'. WoogleUtil::getMsg('SearchTimePeriod') .'</div><table>
			<tr><td>' . WoogleUtil::getMsg('SearchTimeFrom') . ' </td><td><input name="changedMin" size="20" value="' . $advanced['changedMin'] . '" /></td></tr>
			<tr><td>' . WoogleUtil::getMsg('SearchTimeUntil') . ' </td><td><input name="changedMax" size="20" value="' . $advanced['changedMax'] . '" /></td></tr>
			</table></div>
			<div><div>'. WoogleUtil::getMsg('FileSize') .'</div><input name="sizeMin" size="3" value="' . $advanced['sizeMin'] . '" /> - <input name="sizeMax" size="3" value="' . $advanced['sizeMax'] . '" />
			<select name="sizeFactor" size="1">
		';
		foreach ($sizeFactors as $sf) {
			$oAdvanced .= '<option value="' . $sf[0] . '"' . (($advanced['sizeFactor'] == $sf[0]) ? ' selected="selected"' : '') . '>' . $sf[1] . '</option>';
		}
		
		$oAdvanced .= '</select></div>';
		if (WoogleUtil::getQueryService()->supportPages()) {
			$oAdvanced .= '<div><div>Number of Pages:</div><input name="pagesMin" size="3" value="' . $advanced['pagesMin'] . '" /> - <input name="pagesMax" size="3" value="' . $advanced['pagesMax'] . '" /></div>';
		}
		
		$sortKeys = $qs->getSortKeys();
		if (count($sortKeys) > 1) {
			$oAdvanced .= '<div><div style="clear:left">' . WoogleUtil::getMsg('ResultsSortBy') . '</div><select name="sortType" size="1">';
			foreach ($sortKeys as $num => $key) {
				$oAdvanced .= '<option value="' . $num . '"' . (($advanced['sortType'] == $num) ? ' selected="selected"' : '') . '>' . $key . '</option>';
			}
			$oAdvanced .= '</selected><input type="checkbox" name="sortDesc" value="true"' . (($advanced['sortDesc'] == 'true') ? ' checked="checked"' : '') . ' /> '. WoogleUtil::getMsg('OrderDescending') . '</div>';
		}
		$oAdvanced .= '<div style="clear:left">' . WoogleUtil::getMsg('ShowCount', $showCountDropdown, $showCountCheckbox) . '</div>';
		*/
		$oAdvanced .= '</div>';
		
		
		$oAcq = '';
		$oAcq2 = '';
		//
		//		Print Woogle Icons
		//		
		WoogleUtil::debug('WoogleSearch: Print Woogle Icons: ' . $this->getSearchString());
		
		// 		Retrieve query metadata (should be *before* executing queryService
		if ($this->advancedCall && WoogleConfig::get('advanced')){
			$iconText = $this->getIcons(); // TODO clean
			if (WoogleConfig::get('advancedIcons')){
				$oAcq .= $iconText;		
			} else {
				$oAcq .= '<br /><br />';
		
			}
		}
		
		//
		//		Print Woogle Boxes
		//		
		if ($this->advancedCall && WoogleConfig::get('advanced')) {
			if (WoogleConfig::get('advancedBox')) {
				$oAcq .= $this->getWikiBox($linker);
			}			
			if (WoogleConfig::get('advancedWikiLink')) {
				//$oAcq2 .= '<div id="woogleCell3">';
				
				if (!$wgUser->isAnon()) {
					$oAcq2 .= $this->getWikiLinkAndWatchNew(); 
				} else {
					$oAcq2 .= $this->getWikiLinkNew();
				}				
				//$oAcq2 .= '</div>';
			}
			/*
			if (WoogleConfig::get('advancedWatch') && !$this->article->mTitle->userIsWatching()) {
				$wgOut->addHTML($this->getWatch());
				//$wgOut->addHTML("<br />");
			}*/
			/*
			if (WoogleConfig::get('advancedDiscussion')) {
				$this->addDiscussion();
				$wgOut->addHTML("<br />");
			}*/
//			$oAcq .='</div>';
		} else {
			
			if (!empty($this->searchString)){
				
				$t 		= new Title();
				$tt 	= $t->newFromDBkey($this->searchString);
				if (!$tt->exists()){
					$createPageText = WoogleUtil::getMsg('WoogleSearchCreatePageTitle');
					$createPage = $linker->makeBrokenLinkObj($tt, $createPageText);	
				// 	$outText = WoogleUtil::getMsg('WoogleSearchCreatePage', $this->searchString);
					$wgOut->addHTML(WoogleUtil::getMsg('WoogleSearchNoPageFound', $this->getSearchString(), $createPage));					
					$wgOut->addHTML("<br />");
					$wgOut->addHTML("<br />");
									
				}
			}			

		} 
		
		
		//
		//		Standard search form
		//
		
		// clear advanced search box if this is native search
		if (WoogleUtil::getQueryService()->isNative()) $oAdvanced = ''; // TODO solve more nicely
		//$oAdvanced = '';
		
		WoogleUtil::debug('WoogleSearch: searchString is currently: ' . $this->searchString );

		//<script src="' . WoogleUtil::getWooglePath() . 'resources/js/scriptaculous/prototype.js" type="text/javascript"></script>
		//<script src="' . WoogleUtil::getWooglePath() . 'resources/js/scriptaculous/scriptaculous.js" type="text/javascript"></script>			
	
//		$oForm = '
//		<div><form name="woogleForm" >
//		<div id="woogleCell2">
//		<input id="autocomplete" name="searchString" value="' . $this->searchString . '" />
//		<input type="submit" value="' . wfMsg('woogleSearchButton') . '" class="woogle_wooglebutton" />' 
// 		. $oAcq . '</div>' . $oAdvanced . '</form></div>' . $oAcq2 ;

 		$defaultInputValue=$this->searchString;

 		$alternativeForm = '
 			<div>
 				<form name="woogleForm" onsubmit="return callSearch();">
		 			<div id="woogleCell2"> 
		 				<input id="autocomplete" name="searchString" value="' . htmlspecialchars($defaultInputValue) . '" class="woogle_search_input" />
		 				<input type="submit" value="' . wfMsg('woogleSearchButton') . '" class="woogle_search_button" />'
		 				. $oAcq	. '
		 			</div>' . $oAdvanced . '
 				</form>
 			</div>';// . $oAcq2 ;


	 	//<input id="advancedButton" type="button" value="' . WoogleUtil::getMsg('AdvancedSearchButton') . '" onclick="advancedEnable()" />
		//<input id="simpleButton" type="button" value="' . WoogleUtil::getMsg('SimpleSearchButton') . '" onclick="advancedDisable()" />
		
		//' . ($wgRequest->getBool('advanced') ? 'advancedEnable();' : 'advancedDisable();') . '
		// Commented out above by HJ
		
		// TODO try
		//$oForm .= '<br style="clear:left"><br />';
		
		// OpenSearch
		$wgOut->addHTML('<link title="' . wfMsg('woogleTitle') . ' Wiki Search" href="' . WoogleUtil::getWooglePath() . 'services/opensearch.php?type=wiki" type="application/opensearchdescription+xml" rel="search" />');		
		
		if ($multipleRepositories){
			$wgOut->addHTML('<link title="' . wfMsg('woogleTitle') . ' Enterprise Search" href="' . WoogleUtil::getWooglePath() . 'services/opensearch.php?type=all" type="application/opensearchdescription+xml" rel="search" />');
		}
		
		//
		//		Processing will stop here if query string is empty
		//		Otherwise, the result page will be built based on the query
		//
		if (empty($this->searchString)) {
			// show empty page without results
			WoogleUtil::debug('WoogleSearch: Exit because of empty search string');
			$wgOut->setPageTitle(wfMsg('woogleStartTitle'));
//			$wgOut->addHTML($oForm);
			$wgOut->addHTML($alternativeForm);
			$wgOut->addHTML('<div id="woogleResults">');
			$wgOut->addHTML("<br /><br />");
			$this->errorMsg();
			$this->helpMsg();
			$stopTime = microtime(true);
			$wgOut->addHTML('<br /><div align="right"><font color="grey"><small><i>Woogle processing time: ' . ($stopTime - $startTime) . '</i></small></font></div>');
			$wgOut->addHTML('</div>');
			WoogleUtil::profileOut(__METHOD__);
			return;
		}
		
		//
		//		Set title and print search form
		//		
		$wgOut->setPageTitle(wfMsg('woogleResultTitle', $this->searchString));
//		$wgOut->addHTML($oForm);
		$wgOut->addHTML($alternativeForm);
		$this->errorMsg();

		//
		//		TODO: Print query refinements
		//

		/*
		if(empty($oRefinements)) {
			//$wgOut->addHTML(wfMsg('woogleNoRefinements'));
		} else {
			$wgOut->addHTML(wfMsg('woogleRefinements') . $oRefinements);
		}
		$wgOut->addHTML( "<br>" );
		*/		

		// we have a query to answer
		// Get request data from, e.g.
		$page = $wgRequest->getInt('page', -1);
		//$browse = true;
		if ($page < 1) {
			$page = 1;
			//$browse = false;
		}

		//
		// Get results
		//
		
		$oResults = '';
		$pageParams = '';
		$sessionHash = $wgRequest->getText('session', md5(mt_rand())); // 2nd parameter = default value
		try {
			$resultCount = $wgRequest->getInt('count', WoogleConfig::get('defaultCount'));
			$resultStart = ($page - 1) * 10; // TODO only if 10 results per page
			foreach ($advanced as $aKey => $aValue) {
				if (is_array($aValue)) {
					foreach ($aValue as $aOption) {
						$pageParams .= (!empty($pageParams) ? '&' : '') . $aKey . '[]=' . $aOption;
					}
				} elseif (!empty($aValue)) {
					$pageParams .= (!empty($pageParams) ? '&' : '') . $aKey . '=' . $aValue;
				}
			}
			// article is set in includes/wooglearcticlesearch. check, why article is not set here
			if (isset($this->article)){
				if ($this->article->exists()) {
					$advanced['excludeId'] = $this->article->getID();
				}
			}
			$sessionParam = 'session=' . $sessionHash;
			$pageParams .= (!empty($pageParams) ? '&' : '') . $sessionParam;
			
			//
			//		Call queryservice
			//
			$startQTime = microtime(true);
			$resultList = $qs->executeKeywordQuery($this->searchString, $resultCount, $resultStart, $advanced, $referer);
			$stopQTime = microtime(true);

			// run hook for query logging addon
			$logMime = '';
			if (isset($advanced[$mapKeys['mimetype']])){
				$logMime = str_replace(chr(10), '', print_r($advanced[$mapKeys['mimetype']], true));
				$logMime = str_replace(' ', '', $logMime);
			}
			$logRepo = '';
			if (isset($advanced[$mapKeys['repository']])){
				$logRepo = str_replace(chr(10), '', print_r($advanced[$mapKeys['repository']], true));
				$logRepo = str_replace(' ', '', $logRepo);
			}
			wfRunHooks('WoogleSearchLogQuery', array($this->searchString . ';HIT:;' . $resultList['numHits'] . ';RES:;' . $resultCount . ';STR:;' . $resultStart . ';REF:;' . $referer . ';AMT:;' . $logMime . ';ARP:;' . $logRepo) );

			//
			//	Fetch correct headline for search result list
			//
			$found = wfMsg('woogleResultsFoundOne');														// one result found
			if ($resultList['numHits'] == 0) $found = $found = wfMsg('woogleResultsFoundNothing');			// no results 
			if ($resultList['numHits'] > 1) $found = wfMsg('woogleResultsFound', $resultList['numHits']);	// more than one
			//$oResults = '<div id="woogleResults"><br /><h1>' . $found . '</h1>';
			//$oResults = '<div id="woogleResults"><br />';
			$oResults = '<div id="woogleResults">';
			
			//style="background-color: #def; padding: 5px;"
			//$oResults .= '<table class="woogle_advanced_nf" style="margin-top: 1em; margin-bottom: 1em"><tr><td>';
			$oResults .= '<table width="100%" class="woogle_advanced_nf" style="margin-top: 1em; margin-bottom: 1em"><tr width="100%"><td align="left" style="background-color: #def; padding: 5px; padding-right: 0px;" width="50%">';
			$oResults .= $found . '</td><td align="right" style="background-color: #def; padding: 5px; padding-left: 0px">' . $oAcq2 . '</td></tr></table>';
			
			//
			//	Iterate and print query results
			//
			$oResults .= $this->printQueryResults($resultList, $sessionParam, $resultStart);
			
			$oResults .= '</div>';
		} catch (Exception $e) {
			$this->handleException($e, $this->searchString);
			$this->showBuiltInSearch();
			WoogleUtil::profileOut(__METHOD__);
			return;
		}

		$maxPages = max(ceil($resultList['numHits'] / $resultCount), 1);
		$offset = 4;

		$oPages = $this->getPageLink('&lt;&lt;', $page, 1, $this->searchString, $pageParams) . ' ' . self::getPageLink('&lt;', $page, max(1, $page - 1), $this->searchString, $pageParams) . ' ';
		for ($i = max(1, $page - $offset); $i <= min($maxPages, $page + $offset); $i++) {
			$oPages .= $this->getPageLink($i, $page, $i, $this->searchString, $pageParams) . ' ';
		}
		$oPages .= $this->getPageLink('&gt;', $page, min($maxPages, $page + 1), $this->searchString, $pageParams) . ' ' . self::getPageLink('&gt;&gt;', $page, $maxPages, $this->searchString, $pageParams);

		$oGoogle = '<a href="';
		$googleUrl = 'http://www.google.com/search?q=';
		$googleUrl .= urlencode($this->searchString);
		$oGoogle = $linker->makeExternalLink($googleUrl, wfMsg('woogleSearchGoogle', $this->searchString));
		//
		//		Print results
		//
		$wgOut->addHTML($oResults);
		//
		//		Print footer
		//
		//Print hidden field containing searched Query, needed for Ajax-Functions - Don't Change!!!!!!
		$wgOut->addHTML('<input type="hidden" id="searchedString" value="'.$this->searchString.'" />');
		
		// Container for "deleted" results
		$wgOut->addHTML( $this->getTrashContainer() );	
		// Print blaettern
		if ($resultList['numHits'] > 0) {
			$wgOut->addHTML( "<br><br>" );
			$wgOut->addHTML('<table class="woogle_advanced"><tr><td><div align="center">' . $oPages . "</div></td></tr></table>");
		}
		// Add new Woogle result
		$addResultHtml = '
			<div style="display: none;" id="addWoogleResult">
				<img height="16" width="16" 
					src="'. WoogleUtil::getImagePath() . 'social_search/add_result.png" 
					alt="'. WoogleUtil::getMsg('WoogleSearchResultAdd').'"
					title="'.WoogleUtil::getMsg('WoogleSearchResultAdd').'" />
					&nbsp;'.WoogleUtil::getMsg('WoogleSearchResultAdd').'
			</div>'		
		; 
		$wgOut->addHTML($addResultHtml);
		
		$wgOut->addHTML("<br /><br />");
		
		// Print Google link
		$wgOut->addHTML($oGoogle);
		// Print Info
		
		$this->helpMsg();
		$stopTime = microtime(true);	
		$wooPTime = $stopTime - $startTime;
		$wooQTime =	$stopQTime - $startQTime;
		$wgOut->addHTML('<div align="right"><font color="grey"><small><i>Woogle processing time: ' . $wooPTime);
		$wgOut->addHTML('&nbsp;/&nbsp;Woogle query time: ' . $wooQTime . '</i></small></font></div>');
		WoogleUtil::perflog($resultList['numHits'] . ';' . $wooPTime . ';' . $wooQTime . 'search', 'query');
		
		WoogleUtil::profileOut(__METHOD__);
	}
	
	/**
	 * Prints help message to the output stream
	 * @return		void
	 */
	private function helpMsg() {
		global $wgOut;
		WoogleUtil::profileIn(__METHOD__);
		//$helpPage = WoogleConfig::get('helpPage');
				$helpPage = SpecialPage::getTitleFor('WoogleSpecialHelp')->getText();
		if (!empty($helpPage)) {
			$wgOut->addHTML('<table class="woogle_advanced_nf" style="margin-top: 1em; margin-bottom: 1em"><tr><td align="center" style="background-color: #def; padding: 5px;">');
			//$wgOut->addHTML('<div style="text-align: center; background: #F7F7F7; border: 4px solid #B0BFE8; padding: 0.5em; clear: both; ">');
			$wgOut->addWikiText(wfMsg('woogleHelp', $helpPage)); // TODO Localize
			//$wgOut->addHTML('</div>');
			$wgOut->addHTML('</td></tr></table>');
			
		}
		WoogleUtil::profileOut(__METHOD__);
	}
	
	/**
	 * Print query results
	 * 
	 * @param $resultList		array, query results returned from query service
	 * @param $sessionParam		string, "session=$sessionHash"
	 * @param $resultStart		int, number of first result to show
	 * @return 					String, html code for result list
	 */
	private function printQueryResults($resultList, $sessionParam, $resultStart) {
		global $wgContLang;
		$html = '';
		$pos = $resultStart + 1;
		
		if (isset($resultList['results'])) {
			//$wikiPath = WoogleMWUtil::getArticleAbsolutePath();
			$wikiPath = WoogleMWUtil::getWikiAbsolutePath();
			
			// add social results form to add new results
			$html .= $this->getNewResultForm();
			
			// iterate results
			foreach ($resultList['results'] as $r) {
	
				WoogleUtil::debug('S_Label: ' 	. $r['label']);
				WoogleUtil::debug('S_URI: ' 	. $r['uri']);
				WoogleUtil::debug('S_Content: ' . $r['content']);
				
				// TODO Hack
				//$r['uri'] = str_replace('file:////trauerweide/users/', 'file:///u:/', $r['uri']);
				
				// make short urls for internal results to avoid https icons (http://octopus13.fzi.de:8080/browse/WOO-249)
				$urx 	= $r['uri'];
				$host  	= $_SERVER['HTTP_HOST'];
				$upos 	= strpos($urx, $host);
				if ($upos > 0) $urx = substr($urx, $upos + strlen($host));
				
				//
				// add click tracking code
				//
				$link_start = '<a href="' . $urx . '"';
				if (WoogleConfig::hasClickTracking()) {
					// Note:
					// Do NOT use this code snippet again without paying attention to the explicit usage of '?'. This could lead to
					// problems (cf. http://octopus13.fzi.de:8080/browse/WOO-315)
					//$url = WoogleUtil::getWooglePath() . 'services/redirect.php?url=' . urlencode($r['uri']) . '&searchString=' . urlencode($this->searchString) . '&position=' . $pos . '&' . $sessionParam;
					$redirect_url =  WoogleUtil::getWooglePath() . 'services/redirect.php';													
					$link_start = '<a href="' . $urx . '" onmousedown="(new Image).src = \''. $redirect_url . '?position=' . $pos . '&amp;url=\' +
						escape(\'' . $r['uri'] . '\') + \'&amp;searchString=\' + escape(\'' . $this->searchString . '\') + \'&amp;' . $sessionParam . '\'; return true;"';
				}
				
				// TODO: refactor invisible tracking
				/*
				<a href="URL" onmousedown="(new Image).src = 'redirect.jsf?position=1&amp;url=' +
						escape('URL') + '&amp;query=' + escape('QUERY'); return true;">TITLE</a>
				
				<a href="URL" class="external " title="URL" rel="nofollow">LABEL</a>
				*/		
				
				//
				// Check if this a result coming from this wiki
				//
				$external = true;
				$isImg = false;
				$isRedirect = false;
				$link_end = ' rel="nofollow" class="external">' . $r['label'] . '</a>'; 
				$internal_info = '';

				if ($wikiResult = stristr($r['uri'], $wikiPath)) {
				//
				//	Internal result
				//
					
					$t = new Title();
					$tt = $t->newFromDBkey($r['label']);
	
					if (isset($tt) && $tt->exists()) {
						//
						//	Wiki page exists
						//
						$external = false;
						$link_end = ' rel="nofollow">' . $r['label'] . '</a>'; 
						
						// temporary handling of redirects
						if ($tt->isRedirect()){
							global $wgOut;
							$isRedirect = true;
							$art = new Article($tt);
							$target = $art->followRedirect()->getPrefixedURL();
							$rtxt = WoogleUtil::getMsg('WoogleSearchResultRedirect');
							$r['content'] = $wgOut->parse($rtxt . ' [[' . $target . ']]');
						}
						
						//$link = '<a href="' . $url . '">' . $r['label'] . '</a>';
						//$internal_info = WoogleUtil::getMsg('WoogleSearchResultRelevance') . ': ' . round($r['relevance'], 2) . ' - ' . WoogleUtil::getMsg('WoogleSearchResultSize') . ': ' . $wgContLang->formatSize($r['size']) . ' - ' . WoogleUtil::getMsg('ResultLastChange') . ' ' . date('d.m.Y H:i', strtotime($tt->getTouched())) . ' - <a href="' . $tt->getEditURL() . '">' . wfMsg('edit') . '</a>';
						//$dat = date_format(DateTime::createFromFormat($r['changed'], 'YmdGis'), 'd.m.Y H:i');
						global $wgUser;
						$timecorrection = $wgUser->getOption( 'timecorrection' );
						$format = $wgUser->getDatePreference();
						$dat = $wgContLang->timeanddate( $r['changed'], true, $format, $timecorrection);
						$internal_info = WoogleUtil::getMsg('WoogleSearchResultRelevance') . ': ' . round($r['relevance'], 2) . ' - ' . WoogleUtil::getMsg('WoogleSearchResultWords') . ': ' . $wgContLang->formatNum($r['sizeWords']) . ' - ' . WoogleUtil::getMsg('ResultLastChange') . ' ' . $dat . ' - <a href="' . $tt->getEditURL() . '">' . wfMsg('edit') . '</a>';
					} else {
						//
						//	No wiki page - might be internal file!
						//
						
						// TODO might be a file
						$external = false;
						$internal_info = '';
						
						if (stripos($r[WoogleFd::$mimetype], 'image' ) === 0){
							//
							//	Internal image
							//
							$isImg = true;
							
							$ftitle = Title::makeTitleSafe( NS_FILE, $r['label'] );
							$lfile = wfLocalFile( $ftitle );
							
							$r['content'] = '&nbsp;' . WoogleUtil::getMsg('WoogleSearchResultWidth') . ': ' . $lfile->getWidth() . 'px ,&nbsp;' . WoogleUtil::getMsg('WoogleSearchResultHeight') . ': ' . $lfile->getHeight() . 'px';
							//$dat = date_format(DateTime::createFromFormat($r['changed'], 'YmdGis'), 'd.m.Y H:i');
							global $wgUser;
							$timecorrection = $wgUser->getOption( 'timecorrection' );
							$format = $wgUser->getDatePreference();
							$dat = $wgContLang->timeanddate( $r['changed'], true, $format, $timecorrection);
							$internal_info = WoogleUtil::getMsg('WoogleSearchResultRelevance') . ': ' . round($r['relevance'], 2) . ' - '.WoogleUtil::getMsg('WoogleSearchResultSize').': ' . $wgContLang->formatSize($r['size']) . ' - ' . WoogleUtil::getMsg('ResultLastChange') . ' ' . $dat; // . ' - <a href="' . $tt->getEditURL() . '">' . wfMsg('edit') . '</a>';
						} else if (stripos($r[WoogleFd::$mimetype], 'pdf' ) != false){
							//
							//	Internal file (pdf)
							//
							if (strlen($r['content']) === 0) $r['content'] = WoogleUtil::getMsg('WoogleSearchResultNoTextExtracted');
							//$dat = date_format(DateTime::createFromFormat($r['changed'], 'YmdGis'), 'd.m.Y H:i');
							global $wgUser;
							$timecorrection = $wgUser->getOption( 'timecorrection' );
							$format = $wgUser->getDatePreference();
							$dat = $wgContLang->timeanddate( $r['changed'], true, $format, $timecorrection);
							$internal_info = WoogleUtil::getMsg('WoogleSearchResultRelevance') . ': ' . round($r['relevance'], 2) . ' - '.WoogleUtil::getMsg('WoogleSearchResultSize').': ' . $wgContLang->formatSize($r['size']) . ' - ' . WoogleUtil::getMsg('ResultLastChange') . ' ' . $dat; // . ' - <a href="' . $tt->getEditURL() . '">' . wfMsg('edit') . '</a>';
						}
						$link_end = ' rel="nofollow">' . $r['label'] . '</a>';						
						
						WoogleUtil::error("Wiki result does not exist in Wiki (but might be a file): " . $r['uri']);
					}
				}

				//
				// Finally generate HTML for result
				//
				
				$link = $link_start . $link_end;
				$html .= '<div class="woogle_result">'; 

				// XXX something new
				// $link = '<a href="'.$urx.'">'.$r['label'].'</a>';				
				
				$tooltip_text = woogleUtil::getMsg('OpenInNewTab', $r['label']);
				// $mouseOverFunc = 'onmouseover="openInNewTab(\''.$tooltip_text.'\');"';
				$mouseOverFunc = 'title="'.$tooltip_text.'"';
				if ($external) {
					//
					//	External result
					//
					
					//$link = $linker->makeExternalLink($url, $r['label']);
					
					//$chtml = strip_tags($r['content']); // TODO redundant?/removes highlighting?
					$chtml = $r['content'];
					if (strlen($chtml) > 1) $chtml = $chtml . '<br />'; // only add break if there is snippet text to show
															
					$html .= 
						'<span '.$mouseOverFunc.'class="woogle_resultlink" >'.$link.'</span><br />' . $chtml .
						'<span class="woogle_source" id="sub' . $pos . '">' .
						   WoogleUtil::getMsg('WoogleSearchResultRelevance') . ': ' . round($r['relevance'], 2) . ' - '.WoogleUtil::getMsg('WoogleSearchResultSize').': ' . $wgContLang->formatSize($r['size']) . '<br />' . 
						   '<img src="' . WoogleUtil::getWooglePath() . 'resources/img/icon/' . $r['mimetype'] . '.gif" width="12" height="12" onerror="this.src=\'' .
							 WoogleUtil::getWooglePath() . 'resources/img/icon/unknown.gif\'" /> <b>' .
							 $r['source'] . '</b>: <span id="url' . $pos . '">' . urldecode($r['uri']) . '</span>
						</span>';							
					//	$oResults .= '<script language="JavaScript"> truncUrl("' . $pos . '"); </script>';
				} else {
					//
					//	Internal result
					//
					
					// do not show icon + url for internal wiki results
					//$html .= 
					//	'<span class="woogle_resultlink">'.$link.'</span><br />' . strip_tags($r['content']) . '<br />
					//	<span class="woogle_source" id="sub' . $pos . '">' . $internal_info . '</span>';							
					$chtml = $r['content'];
					if ((strlen($chtml) > 1) && (!$isRedirect)) $chtml = $chtml . '<br />'; // only add break if there is snippet text to show
					if ($isImg){
						$imageSrc = urldecode( $r['uri'] );
						$imageSrc = utf8_encode( $imageSrc );
						$html .= 
							'<span '.$mouseOverFunc.' class="woogle_resultlink">'.$link.'</span><br /><img src="' . $imageSrc . '" height="50px">'. $chtml.
							'<span class="woogle_source" id="sub' . $pos . '">' . $internal_info . '</span>';							
					} else {
						$html .= 
							'<span '.$mouseOverFunc.'class="woogle_resultlink">'.$link.'</span><br />' . $chtml .
							'<span class="woogle_source" id="sub' . $pos . '">' . $internal_info . '</span>';	
					}
					
				}
				
				//
				// Add result comments
				//
				
				// $comments contains an Array of comments. At the moment the key "comment_text" is necessary
				// $comments = false;
				$comments = array();
				$val1 = array(
					'comment_text'	=> 'comment von val1'					
				);
				$val2 = array(
					'comment_text'	=> 'comment von val2'
				);
				$comments[] = $val1;
				$comments[] = $val2;
				if( WoogleConfig::get('socialResults')) {								
					$html .= self::getChangeButtons($pos, $comments).'<br /></div>' . "\n";
				} else {
					$html .= '</div><br/>';					
				}		

				
				$pos++;
			}			
		} else {
			// social results form to add new results
			$html .= $this->getNewResultForm();
		}	
		return $html;
	}

	
	/**
	 * Prints the "TrashContainer"
	 */
	private function getTrashContainer( $content = '' ) {
		
		if( $content == '' ) {
			$display = 'style="display:none"';			
		} else {
			$display = 'style="display:block"';
		}
		$header = '
			<div id="woogleResultsTrashHeader" align="center" '.$display.'>
				<span class="trashHide" style="display:none;">'.WoogleUtil::getMsg('WoogleSearchTrashHide').'</span>
				<span class="trashShow">'.WoogleUtil::getMsg('WoogleSearchTrashShow').'</span>
			</div>';
		
		$contentFrame = '<div id="woogleResultsTrash" style="display: none">'.$content.'</div>';
		
		return $header.$contentFrame;
		
	}
	
	/**
	 * Social Results form for adding new results to search
	 * 
	 * @return 		string, html code of the container
	 */
	private function getNewResultForm() {
		$html = '
			<div id="newResult" class="woogle_result" style="padding: 5px; margin-bottom: 10px; border: 2px dotted; display: none;" >
				<span class="woogle_resultlink">'.WoogleUtil::getMsg('WoogleSearchAddResultHead').'</span><br/>
				<table cellpadding="5">
				<colgroup>
					<col width="200">
					<col width="350">
					<col width="150">				
				</colgroup>
				<tr>
					<td>'.WoogleUtil::getMsg('WoogleSearchAddResultUrl').':</td> 
					<td><input style="width:335px;" type="text" class="url" /><input type="button" style="float: right; height: 25px; width: 25px; display:none;" value="X" class="change_url" /></td>
					<td><span style="display: none;" class="error error_url">'.WoogleUtil::getMsg('WoogleSearchAddResultUrlError').'</span></td>
				</tr><tr>
					<td>'.WoogleUtil::getMsg('WoogleSearchAddResultTitle').':</td>
					<td><input style="width:335px;" type="text" class="title" /></td>
					<td><span style="display: none;" class="error error_title">'.WoogleUtil::getMsg('WoogleSearchAddResultTitleError').'</span></td>
				</tr><tr>				
					<td valign="top">'.WoogleUtil::getMsg('WoogleSearchAddResultDescription').':</td>
					<td><textarea style="width:335px;"  class="description"></textarea></td>
					<td><span style="display: none;" class="error error_description">'.WoogleUtil::getMsg('WoogleSearchAddResultDescriptionError').'</span></td>
				</tr><tr>					
					<td><input class="save" type="submit" value="'.WoogleUtil::getMsg('WoogleSearchAddResultSave').'" /></td>
					<td><input class="cancel" type="submit" value="'.WoogleUtil::getMsg('WoogleSearchAddResultDelete').'" /></td>
					<td><input class="result_id" type="hidden" value="false" /></td>				
				</tr>
				</table>
			</div>';				
		return $html;		
	}
	/**
	 * @param integer 		the Original pos of the result without ranking
	 * @param integer		0 or an higher integer (if 0, the result will be handeld as not ranked)
	 * @return html-code for special functions
	 */
	private static function getChangeButtons($originalPos, $comments = false, $rankedPos = 0, $isTrash = 'false') {
		if( WoogleConfig::get('socialResults')) {
			$html = '
				<div align="right">
					<table cellspacing="0" cellpadding="2"><tr>
					<td><img style="display: none;" class="result_comment"
						height="16" width="16"
						src="'. WoogleUtil::getImagePath() . 'social_search/comment_result.jpg"
						alt="'.WoogleUtil::getMsg('WoogleSearchResultComment').'"
						title="'.WoogleUtil::getMsg('WoogleSearchResultComment').'"/>
					</td><td><img style="display: none;" class="result_remove"
						height="16" width="16"
						src="'. WoogleUtil::getImagePath() . 'social_search/remove_result.png"
						alt="'.WoogleUtil::getMsg('WoogleSearchResultRemove').'"
						title="'.WoogleUtil::getMsg('WoogleSearchResultRemove').'"/>
					</td><td><img style="display: none;" class="result_restore"
						height="16" width="16"
						src="'. WoogleUtil::getImagePath() . 'social_search/restore_result.png"
						title="'.WoogleUtil::getMsg('WoogleSearchResultRestore').'"
						alt="'.WoogleUtil::getMsg('WoogleSearchResultRestore').'"/>		
					</td><td><img style="display: none;" class="result_promote"
						height="16" width="16"
						src="'. WoogleUtil::getImagePath() . 'social_search/promote_result.png"
						title="'.WoogleUtil::getMsg('WoogleSearchResultPromote').'"
						alt="'.WoogleUtil::getMsg('WoogleSearchResultPromote').'"/>
					</td><td><img style="display: none;" class="result_degrade"
						height="16" width="16"
						src="'. WoogleUtil::getImagePath() . 'social_search/degrade_result.png"
						title="'.WoogleUtil::getMsg('WoogleSearchResultDegrade').'"
						alt="'.WoogleUtil::getMsg('WoogleSearchResultDegrade').'"/>
					</td></tr></table>	
					<input type="hidden" class="originalPos" value="'.$originalPos.'" />
					<input type="hidden" class="rankedPos" value="'.$rankedPos.'" />
					<input type="hidden" class="isTrash" value="'.$isTrash.'" />
				</div>';
			if( $comments ) {
				$html .= '<div class="result_comments" style="display:none">';
				foreach( $comments as $comment) {
					$html .= '<div class="comment">'.$comment['comment_text'].'</div>';	
				}				
				$html .= '</div>';
			} else {
				$html .= '<div class="result_comments" style="display:none"></div>';
			}			
			$html .= self::addCommentForm();
		} else {
			$html = '';
		}
		return $html;
	}
	
	private static function addCommentForm() {
		$html = '<div class="new_comment" style="display: none" >
					<input class="std_comment" type="hidden" value="'.WoogleUtil::getMsg('WoogleSearchCommentText').'" />
					<textarea class="comment" style="display:block;">'.WoogleUtil::getMsg('WoogleSearchCommentText').'</textarea>
					<input class="add_comment" disabled="disabled" type="submit" value="'.WoogleUtil::getMsg('WoogleSearchCommentAdd').'" />
					<input class="cancel" type="submit" value="'.WoogleUtil::getMsg('WoogleSearchCommentCancel').'" />		
				</div>';						
		return $html;	
	}
	
	public static function getNewChangeButtons(){
		// first Param should be 0 , not 1, because after inserting it gets increased
		return self::getChangeButtons(0,0,'false');		
	}
	/**
	 * Prints error messages from the error array to the output stream
	 * @return		void
	 */
	private function errorMsg() {
		global $wgOut;
		WoogleUtil::profileIn(__METHOD__);
		if (count ($this->error) > 0){
		
			$wgOut->addHTML('<div style="text-align: center; background: #F7F7F7; border: 4px solid #B0BFE8; padding: 0.5em; clear: both; ">');
			foreach ($this->error as $e) {
				$wgOut->addHTML('<p class="error">' . $e . '</p>');
			}
			$wgOut->addHTML('</div>');
		}
		
		WoogleUtil::profileOut(__METHOD__);
		
	}
	
	private function getErrorMsg(){
		$res = '';
		foreach ($this->error as $e) {
			$res .= $e . '/';
		}
		return $res;
	}
	
	/*
	 * If the user is logged in, this function shows one Textfield and 2Buttons:
	 * field1: searched String
	 * button1: Create new Wiki page
	 * button2: watch this search
	 */
	private function getWikiLinkAndWatch(){
		WoogleUtil::profileIn(__METHOD__);
		global $wgUser;

		$html = '
			<table class="woogle_advanced_nf" width="100%">
			<tr>
				<td align="left">
					<form action="' . WoogleMWUtil::getWikiPath() . 'index.php" method="get" style="display:inline">
						<input name="title" value="' . $this->searchString . '" size="10" />
						<input type="hidden" name="action" value="edit" />
						<input type="submit" value="' . wfMsg('woogleWikiLink') . '" />
					</form>
					&nbsp;&nbsp;
					<form action="' . WoogleMWUtil::getWikiPath() . 'index.php" method="get" style="display:inline">
						<input type="hidden" name="title" value="' . $this->article->getTitle()->getPrefixedText() . '" />
						<input type="hidden" name="action" value="watch" />
						<input type="submit" value="' .  wfMsg('woogleWatch') . '" />
					</form>		
				</td>
			</tr>
			</table>';
			
			return WoogleUtil::profileOut(__METHOD__ , $html);
	}
	
	private function getWikiLinkAndWatchNew(){
		WoogleUtil::profileIn(__METHOD__);
		global $wgUser;

		$html = '
					<form action="' . WoogleMWUtil::getWikiPath() . 'index.php" method="get" style="display:inline">
						<input type="hidden" name="title" value="' . $this->searchString . '" />
						<input type="hidden" name="action" value="edit" />
						<input type="submit" value="' . wfMsg('woogleWikiLink') . '" />
					</form>&nbsp;<form action="' . WoogleMWUtil::getWikiPath() . 'index.php" method="get" style="display:inline">
						<input type="hidden" name="title" value="' . $this->article->getTitle()->getPrefixedText() . '" />
						<input type="hidden" name="action" value="watch" />
						<input type="submit" value="' .  wfMsg('woogleWatch') . '" />
					</form>		
			';
			
			return WoogleUtil::profileOut(__METHOD__ , $html);
	}
	
	/*	
	 * If the user is not logged in, this function shows one textfield and one button:
	 * field1: searched String
	 * button1: Create new Wiki page 
	 */
	private function getWikiLink() {
		WoogleUtil::profileIn(__METHOD__);

		$html = '
			<table class="woogle_advanced_nf" width="100%">
			<tr>
				<td align="left">
					<form action="' . WoogleMWUtil::getWikiPath() . 'index.php" method="get" style="display:inline">
						<input name="title" value="' . $this->searchString . '" size="10" />
						<input type="hidden" name="action" value="edit" />
						<input type="submit" value="' . wfMsg('woogleWikiLink') . '" />
					</form>
				</td>
			</tr>
			</table>';
			
		return WoogleUtil::profileOut(__METHOD__ , $html);
	}
	
	private function getWikiLinkNew() {
		WoogleUtil::profileIn(__METHOD__);

		$html = '
			<form action="' . WoogleMWUtil::getWikiPath() . 'index.php" method="get" style="display:inline">
						<input type="hidden" name="title" value="' . $this->searchString . '" />
						<input type="hidden" name="action" value="edit" />
						<input type="submit" value="' . wfMsg('woogleWikiLink') . '" />
					</form>';
			
		return WoogleUtil::profileOut(__METHOD__ , $html);
	}
	
	/**
	 * 
	 * @return 	string, ?
	 */
	private function getWatch() {
		WoogleUtil::profileIn(__METHOD__);
		$html = '
			<table class="woogle_advanced">
				<tr>
					<td align="right">
						<form action="' . WoogleMWUtil::getWikiPath() . 'index.php" method="get">
							<input type="hidden" name="title" value="' . $this->article->mTitle . '" />
							<input type="hidden" name="action" value="watch" />
							<input type="submit" value="' .  wfMsg('woogleWatch') . '" />
						</form>
					</td>
				</tr>
			</table>';
			
		return WoogleUtil::profileOut(__METHOD__, $html);
	}
	
	/**
	 * Must be called before issueing the actual query to avoid getting wrong metadata
	 *
	 *
	 * @return 	string, ?
	 */
	private function getIcons() {
		WoogleUtil::profileIn(__METHOD__);
		
		// TODO BUG: Article is not set for Special:Woogle
		//$watchers = WoogleMWUtil::getNumWatches($this->article->getTitle()); // method sig changed!
		$iLog = '';
		
		$searchers	= 0;
		$queries	= 0;
		
		$pageRatio;
		$recency;
		$infoNeed = WoogleUtil::getQueryManager()->getInformationNeed($this->searchString);
		//$avgPages = WoogleUtil::getQueryManager()->getAvgPages();

		/*
		echo "<!-- ============\n<br>\n<br>\n<br>InfoNeed: ";
		print_r($infoNeed);
		echo "============\n<br>\n<br>\n<br>";
		*/
		
		$pIcon = '';		// priority icon
		$rIcon = '';		// recency icon
		$qIcon = '';		// result quality icon
		
		if ($infoNeed != false) {
			
			$searchers		= $infoNeed['sizeQueryUsers'];
			$queries		= $infoNeed['needExecutions'];
			//$queries--;		// substract current query
			
			$yText 			= '';

			/*
			// page ration not really used currently
			if ($avgPages == 0) {
				$pageRatio = 1;
			} else {
				$pageRatio = ((float) $infoNeed['needExecutionsWithFurtherPages']) / $avgPages;
			}
			*/
			//$recency = $infoNeed['recency'];

			
			//
			// 		Check recency icon
			//
			
			 // TODO check date format native/remote
			$recentQueries 	= 0;
			$relatedQueries = '';
			
			$days30 = time() - (60*60*24*30); // now - 30 days
			
			/*
			if ($days30 < $infoNeed['lastDate']){
				$relatedQueries .= $infoNeed['needString'];
				$recentQueries++;
			}*/
			
			WoogleUtil::debug('Calling getRelatedQueries with ' . $this->searchString);
			$relatedq = WoogleUtil::getQueryManager()->getRelatedQueries($this->searchString, 20, false);
			WoogleUtil::debug('Found ' . count($relatedq) . ' related searches for ' . $this->searchString);
		//	echo 'Found ' . count($relatedq) . ' related searches for ' . $this->searchString;
			$first = true;
			foreach($relatedq as $rquery){
				if ($days30 < $rquery['lastDate']){ 
					$recentQueries++;
					if (!$first) $relatedQueries .= ', '; 
					$first = false;
					$relatedQueries .= $rquery['needString'];
				}
			}
			
			
			if ($recentQueries > 1){
				$rText = WoogleUtil::getMsg('WoogleSearchRelatedSearches', $recentQueries);
				$rIcon = '<br /><img src="' . WoogleUtil::getImagePath() . 'recency_1.gif" alt="' . $rText . '" title="' . $relatedQueries . '" />&nbsp;' . $rText;
			} else if ($recentQueries == 1){
				$rText = WoogleUtil::getMsg('WoogleSearchRelatedSearchOne');
				$rIcon = '<br /><img src="' . WoogleUtil::getImagePath() . 'recency_1.gif" alt="' . $rText . '" title="' . $rText . '" />&nbsp;' . $rText;
			}
			
			$iLog .= 'S:;' . $searchers . ';Q:;' . $queries . ';RQ:;' . $recentQueries;
			
			//
			// 		Check result quality icon
			//

			
			// FEW HITS heuristics
			
			$y = ((float) $infoNeed['needExecutionsWithoutResultClick'])/ ((float) $infoNeed['needExecutions']);
			
			//$z = 1-(((float)$infoNeed['resultClicks'])/((float)$infoNeed['needExecutions']));
			
			$val_few = $y;
			//$val_few = $y + $z;
			
			if ($val_few > 0.2){ // was 1.4
//				echo "!!VALFEW MATCH!!";
				$iLog .= ';VLFW:;' . $val_few;
				$yText .= WoogleUtil::getMsg('WoogleSearchNoAppropriate');
			} else {
				$iLog .= ';VLFWx:;' . $val_few;
			}
//			echo "VAL_FEW: " . $val_few;
			
			// DIFFERENT CLICKS/SCATTERED heuristics
			
			$val_diff = ((float)$infoNeed['resultClicks'])/((float)$infoNeed['needExecutions']);
			if ($val_diff > 1.2){
//				echo "!!DIFFCLICKS MATCH!! ";
				$iLog .= ';DFFC:;' . $val_diff;
				$yText .= WoogleUtil::getMsg('WoogleSearchScattered');				
			} else {
				$iLog .= ';DFFCx:;' . $val_diff;
			}
//			echo "VAL_DIFF: " . $val_diff;
			
			
			// TOO MUCH BROWSING heuristics
			
			$a = ((float)$infoNeed['needExecutionsWithFurtherPages'])/((float)$infoNeed['needExecutions']);
			
			$val_much = $a * $infoNeed['needExecutionsWithFurtherPagesAveragePages'];
			
			if ($val_much > 0.2){
//				echo "!!MUCHBROWSE MATCH!!";
				$iLog .= ';MBRW:;' . $val_much;
				$yText .= WoogleUtil::getMsg('WoogleSearchManyInvestigated');				 	
			} else {
				$iLog .= ';MBRWx:;' . $val_much;
			}
//			echo "VAL_MUCH: " . $val_much;
			
			if ($yText != ''){
				$qIcon = '<br /><img src="' . WoogleUtil::getImagePath() . 'quality_1.gif" alt="' . $yText . '" title="' . $yText . '" />&nbsp;' . $yText;
//				echo "YTEXT: " . $yText;
			} else {
//				echo "YTEXT EMPTY: " . $yText;
			}
			
		} else {
			//$pageRatio = 1;
			//$recency = 0;		// first time this query is executed - no previous query
		}
		
		
//		echo " -->";
		
		$qText = WoogleUtil::getMsg('RedLinkResultTextFirst');
		if ($queries > 1) $qText = WoogleUtil::getMsg('RedLinkResultText', $queries, $searchers);
		if ($searchers == 1){
			if ($queries == 1){
				$qText = WoogleUtil::getMsg('RedLinkResultTextOneOne');
			} else {
				$qText = WoogleUtil::getMsg('RedLinkResultTextOneUser', $queries);
			}
		}
		
		if ($queries < 3) $qIcon = '';		// reset result quality

		//
		// 		Select priority icon
		//

		if ($searchers > 5){
			// priority 1
			//$img = WoogleUtil::getImagePath() . 'importance_1.gif';
			$img = WoogleUtil::getImagePath() . 'user_3.gif';
			$pIcon = '<img src="' . $img . '" alt="'. $qText. '" title="' . $qText . '" />&nbsp;'  . $qText;
		} else if ($searchers > 2){
			// priority 2
			//$img = WoogleUtil::getImagePath() . 'importance_2.gif';
			$img = WoogleUtil::getImagePath() . 'user_2.gif';
			$pIcon = '<img src="' . $img . '" alt="'. $qText. '" title="' . $qText . '" />&nbsp;' . $qText;
		} else{
			// priority 3 - no icon
			//$itext .= "PRIO03";
			$pIcon = $qText;
		}
		
		//$out = '<small>' . $pIcon . $rIcon . $qIcon . '</small>';
		//$out = $pIcon . $rIcon . $qIcon;
		
		$out = '<table class="woogle_advanced_nf" style="margin-top: 1em; margin-bottom: 1em"><tr>
			<td style="background-color: #def; padding: 5px;">'.
			$pIcon . '<small>' . $rIcon . $qIcon . '</small>' . 
		 '</td></tr></table>';
		
		
		// run hook for instrumentation addon
		wfRunHooks('WoogleSearchIcons', array($iLog));
		
		
//		$icon = array(
//			array(
//				'img' => WoogleRedLinks::getImportanceIcon($queries),
//				'tooltip' => $queries . ' ' . WoogleUtil::getMsg('RedLinksQueries'),
//				),
//			array(
//				'img' => WoogleRedLinks::getRecencyIcon($recency, WoogleUtil::getQueryManager()->getAvgRecency()),
//				'tooltip' => date('j M Y', $recency),
//				),
//			array(
//				'img' => WoogleRedLinks::getUserIcon($watchers, $searchers),
//				'tooltip' => $watchers . ' ' . WoogleUtil::getMsg('RedLinksWatchers').', ' . $searchers . ' ' . WoogleUtil::getMsg('RedLinksSearchers'),
//				),
//			array(
//				'img' => WoogleRedLinks::getQualityIcon($pageRatio),
//				'tooltip' => WoogleRedLinks::getQualityText($pageRatio),
//				),
//			);
			
		/*$stuff = '
<table class="woogle_advanced">
<tr><td width="1%">';*/
		//$stuff = '<br /><br />';
		
		// TODO
		//$stuff .= $itext . " / ITEXT";
		
//		foreach ($icon as $i) {
//			$stuff .= '<img src="' . $i['img'] . '" title="' . $i['tooltip'] . '" />';
//		}
//		if ($queries > 0){
//			$stuff .= ' ' . WoogleUtil::getMsg('RedLinkResultText', $queries, $searchers);
//		} else {
//			$stuff .= ' ' . WoogleUtil::getMsg('RedLinkResultTextFirst');
//		}
		/*$stuff .= '</td></tr>
</table>
';*/
		

		
		return WoogleUtil::profileOut(__METHOD__, $out);
	}
	
	/**
	 * Adds discussion panel to the output stream
	 * @return		void
	 */
	private function addDiscussion() {
		global $wgOut;
		WoogleUtil::profileIn(__METHOD__);
		$wgOut->addHTML('
			<table class="woogle_advanced">
			<tr><td width="100%">');
		$wgOut->addWikiText(WoogleUtil::getMsg('DiscussionLink', $this->article->getTitle()->getTalkPage()->getPrefixedText()));
		$wgOut->addHTML('</td></tr>
			</table>
		');
		WoogleUtil::profileOut(__METHOD__);
	}
	
	/**
	 * Get WikiBox html
	 * 
	 * @param	$linker
	 * @return	String, html code
	 */
	private function getWikiBox($linker) {
		global $wgOut;
		WoogleUtil::profileIn(__METHOD__);
		
		$res = '';
				
		// Get wiki page for
		$woogleDiscussString = WoogleUtil::getNamespaces(WOO_NS_TALK); 
		$t 		= new Title();		
		$tt 	= $t->newFromDBkey("Woogle:" . $this->searchString);
		$ttt 	= $t->newFromDBkey($woogleDiscussString. $this->searchString);
		
		$describe_empty = true;
		$pageExists = $tt->exists();	
		$descriptionText = wfMsg('woogleBoxEmpty');		
		if ($pageExists) {
			$article = new Article($tt);
			$content = $article->getContent();
			if (!empty($content)) {				
				$descriptionText = $content;
				$describe_empty = false;
			}
		}
		
		$discussion_empty = true;
		$pageExists = $ttt->exists();		
		$discussionText = wfMsg('woogleBoxEmpty');
		if ($pageExists) {
			$article = new Article($ttt);
			$content = $article->getContent();
			if (!empty($content)) {				
				$discussionText = $content;
				$discussion_empty = false;
			}
		}

		$wikiDiscussText = WoogleUtil::getMsg('WoogleSearchDiscuss');
		$wikiDescribeText = WoogleUtil::getMsg('WoogleSearchDescribe');	

		
		// uses this code, if Ajax inPlaceEditing is enabled ..			
		if (WoogleConfig::get('inPlaceEditing')){
			$wikiDiscuss = '<a class="%s discuss">'.$wikiDiscussText.'</a>';					
			$wikiDescribe = '<a class="%s describe">'.$wikiDescribeText.'</a>';
			$wikiEdit = '<a class="edit">'.WoogleUtil::getMsg('InfoBoxEdit').'</a>';
			
			if( $discussion_empty) {
				$wikiDiscuss = sprintf( $wikiDiscuss, 'new');
			}	else {
				$wikiDiscuss = sprintf( $wikiDiscuss, '');
			}
			if( $describe_empty) {
				$wikiDescribe = sprintf( $wikiDescribe, 'new');
			} else {
				$wikiDescribe = sprintf( $wikiDescribe, '');
			}
			
			$styleWrap = 'style="display: %s ; "' ;			
			
			if($describe_empty) {
				$description_text_style = sprintf( $styleWrap, 'none');
				$no_description_style = sprintf( $styleWrap, 'block');
				$description_style = sprintf( $styleWrap, 'none');	
				$description_title_style = sprintf( $styleWrap, 'none');			
			} else {
				$description_text_style = sprintf( $styleWrap, 'block');
				$no_description_style = sprintf( $styleWrap, 'none');
				$description_style = sprintf( $styleWrap, 'block');
				$description_title_style = sprintf( $styleWrap, 'block');								
			}
			
			$res .= '
				<table class="woogle_advanced_nf searchDescriptionAndDiscussion">
					<tr class="no_description" '.$no_description_style.'>
						<td>'.WoogleUtil::getMsg('WoogleSearchDescribeOrDiscuss', $wikiDescribe, $wikiDiscuss) .'</td>
					</tr>		
					<tr class="description_title" '.$description_title_style.'">
						<th>' . wfMsg('woogleBoxTitle') . '</th>
					</tr>			
					<tr class="write_description" style="display:none">
						<td>
							<textarea class="inPlaceEditingTextarea" >'.$descriptionText.'</textarea>
							<input type="button" class="save" value="'.WoogleUtil::getMsg('WoogleSaveDescription').'">				
							<input type="button" class="cancel" value="'.WoogleUtil::getMsg('WoogleCancelDescription').'">
							<input type="hidden" class="title" value="'.$tt.'"/>
							<input type="hidden" class="type" value="description"/>
						</td>
					</tr>
					<tr class="write_discussion" style="display:none">
						<td>						
							<textarea class="inPlaceEditingTextarea">'.$discussionText.'</textarea>
							<input type="button" class="save" value="'.WoogleUtil::getMsg('WoogleSaveDiscussion').'" />
							<input type="button" class="cancel" value="'.WoogleUtil::getMsg('WoogleCancelDiscussion').'" />							
							<input type="hidden" class="title" value="'.$ttt.'"/>
							<input type="hidden" class="type" value="discussion"/>
						</td>
					</tr>
					<tr class="description_text" '.$description_text_style.'>
						<td class="inPlaceEditingDescriptionText" >							
							'.$wgOut->parse($descriptionText).'							
						</td>
					</tr>
					<tr class="description" '.$description_style.'>
						<td>' . $wikiEdit . ' - ' . $wikiDiscuss . '</td>
					</tr>
				</table>';			
		// .. in this code if Ajax inPlaceEditing isn't enabled
		} else {
			$wikiDiscuss = $linker->makeBrokenLinkObj($ttt, $wikiDiscussText, "action=edit"); // WoogleUtil::getMsg('DiscussionLink')		
			$wikiDescribe = $linker->makeBrokenLinkObj($tt, $wikiDescribeText, "src=woo");
			
			$wikiEdit = $tt->getEditURL();	// TODO: this does not make much sens?
			$wikiEdit = $linker->makeBrokenLinkObj($tt, WoogleUtil::getMsg('InfoBoxEdit'), "src=woo");	
		
			// no woogle page yet	
			if ($describe_empty){
				$res .= '
					<table class="woogle_advanced_nf">
					<tr>
						<td>'.WoogleUtil::getMsg('WoogleSearchDescribeOrDiscuss', $wikiDescribe, $wikiDiscuss) .'</td>
					</tr>
					</table>';			
			// woogle page exists - show
			} else {
				$res .= '
					<table class="woogle_advanced_nf">
					<tr>
						<th>' . wfMsg('woogleBoxTitle') . '</th>
					</tr><tr>
						<td class="inPlaceEditingDescriptionText" >
							'.$wgOut->parse($descriptionText).'
						</td>
					</tr><tr>
						<td>' . $wikiEdit . ' - ' . $wikiDiscuss . '</td>
					</tr>
					</table>';
			}
		}

		WoogleUtil::profileOut(__METHOD__);
		return $res;
	}
	

	
	/**
	 * TODO document properly
	 * 
	 * @param $title			link title
	 * @param $currentPage		int, number of page currently displayed
	 * @param $page				int, page number to link to (?)
	 * @param $url				link target (query)
	 * @param $params			?
	 * @return					string, 
	 */
	private function getPageLink($title, $currentPage, $page, $url, $params = '') {
		WoogleUtil::profileIn(__METHOD__);
		
		if (!empty($params)) {
			$params = '&' . $params;
		}
		
		if ($page != $currentPage) {
			$html = '<a href="' . WoogleUtil::getWoogleLinkPrefix() . $url . '?page=' . $page . $params . '">' . $title . '</a>';
			return WoogleUtil::profileOut(__METHOD__, $html);
		} else {
			$html = '<b>' . $title . '</b>';
			return WoogleUtil::profileOut(__METHOD__, $html);
		}
	}
	
	/**
	 * Display error page instead of search results
	 * 
	 * @param $e			exception
	 * @param $par			string, exception message
	 * @return 				void
	 */
	private function handleException($e, $par) {
		global $wgOut;
		WoogleUtil::profileIn(__METHOD__);
		
		$wgOut->setPageTitle(WoogleUtil::getMsg('ErrorTitle'));
		$this->error[] = WoogleUtil::getMsg('NotAvailable', $par, SpecialPage::getTitleFor('Search')->getPrefixedText());
		$this->errorMsg();
		$this->helpMsg();
		WoogleUtil::sendErrorMail('WoogleSearch failed' , 'WoogleSearch failed with error: ' . $this->getErrorMsg());
		WoogleUtil::error($this->getErrorMsg());
		$msg = '<br><br><a href="#" onclick="document.getElementById(\'exception\').style.display = \'block\'">Display Exception</a>';
		$msg .= '<div id="exception" style="display:none">Error: ' . $e->getMessage() . "<br><pre> " . $e->getTraceAsString() . "</pre></div>";// . "<br>Body: " . $e->getBody();
		$wgOut->addHTML($msg);
		
		WoogleUtil::profileOut(__METHOD__);
	}
	
	private function showBuiltInSearch(){
		global $wgRequest, $wgUser;
		WoogleUtil::profileIn(__METHOD__);
		
		// TODO Temp Duplication
		if ($this->advancedCall) {
			// called in advanced mode
			$sstring = urldecode($this->article->getTitle()->getText());
			//$this->searchString = strtolower($sstring);	// normalize Woogle pages
			$this->searchString = mb_strtolower($sstring);	// normalize Woogle pages
		}
		
		$sps = new SpecialSearch($wgRequest, $wgUser);
		$sps->showResults($this->getSearchString());
		//$sps->showResults('test');
		WoogleUtil::profileOut(__METHOD__);
	}
	
}