<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2009 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik and der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

/**
 * This is a data structure representing content items ("documents") to
 * be written to the index, independent of a particular indexing library
 */
class WoogleDocument {
	//for these fields c.f. also WoogleFd.php
	private $id;			// long id for index (e.g. urn:mediawiki:http://localhost/wiki/99
	private $shortId;		// short id (e.g. article id for index job - 99
	private $uri;			// ..
	private $label;			// ..
	private $content;		// ..
	private $mimetype;
	private $created; // MediaWiki concatenated string timestamp (YYYYMMDDHHMMSS) 
	private $changed; // MediaWiki concatenated string timestamp (YYYYMMDDHHMMSS) 
	//private $source;
	//private $repository;
	private $pages = -1;
	private $size;	// size in bytes
	//private $sizeChars = -1;
	//private $sizeWords = -1;
	//private $sizeLines = -1;
	//private $scope;
	//private $genre;
	//private $language;
	
	public function getId(){
		return $this->id;
	}
	
	public function setId($id){
		$this->id = $id;
	}
	
	public function getShortId(){
		return $this->shortId;
	}
	
	public function setShortId($shortId){
		$this->shortId = $shortId;
	}
	public function getUri(){
		return $this->uri;
	}
	
	public function setUri($uri){
		$this->uri = $uri;
	}
	
	public function getLabel(){
		return $this->label;
	}
	
	public function setLabel($label){
		$this->label = $label;
	}
	
	public function getContent(){
		return $this->content;
	}
	
	public function setContent($content){
		$this->content = $content;
	}
	
	public function getCreated(){
		return $this->created;
	}
	
	public function setCreated($created){
		$this->created = $created;
	}

	public function getChanged(){
		return $this->changed;
	}
	
	public function setChanged($changed){
		$this->changed = $changed;
	}
	
	public function getMimetype(){
		
		if (isset($this->mimetype)){
			return $this->mimetype;
		} else {
			return WoogleQueryServiceCommon::getMimetype();	// TODO default for wiki pages	
		}
			
	}
	
	public function setMimetype($mimetype){
		$this->mimetype = $mimetype;
	}
	
	public function getSource(){
		return WoogleQueryServiceCommon::$source;
	}
	
	public function getRepository(){
		return WoogleQueryServiceCommon::getRepository();
	}
	
	public function getPages(){
		return $this->pages;	
	}
	
	public function setPages($pages){
		$this->pages = $pages;	
	}
	
	public function getSize(){
		if (isset($this->size)){
			return $this->size;
		} else {
			return $this->getSizeChars();	// default for wiki pages	
		}
	}
	
	public function getSizeChars(){
		return strlen($this->getContent());
	}
	
	public function getSizeWords(){
		$ax = explode(' ', $this->getContent());
		return count($ax);
	}
	
	public function getSizeLines(){
		$ax = explode("\n", $this->getContent());
		return count($ax);
	}
	
	public function setSize($size){
		$this->size = $size;
	}
	
	public function getScope(){
		return WoogleQueryServiceCommon::$scope;
	}
	
	public function getGenre(){
		return WoogleQueryServiceCommon::$genre;
	}
	
	public function getLanguage(){
		return WoogleQueryServiceCommon::$language;
	}
	
}

?>