<?php

require_once('Woogle_TestCase.php');
require_once('MockUser.php');

class WoogleQueryManagerNativeTest extends Woogle_TestCase{
	
    protected function setUp(){
//		echo "\n Calling setUp()";
    	$this->prepareTestDb();
		$this->prepareTestIndex();
		$this->prepareQueryService();	
    	$this->qm = WoogleUtil::getQueryManager();
   	  	$this->prepareQueryTestData();
    }
	
	protected function tearDown(){
//		echo "\n Calling tearDown()";
		$this->removeQueryTestData();
		unset($this->qm);
	}
	
	/* Test methods */
	
	public function testHandleQuery(){

		// plain test
		$in = $this->qm->getInformationNeed('Test');	
		$this->assertEquals($this->num_test_queries, $in['needExecutions']);
		
		$this->qm->handleQuery('Test', 20);
		$this->qm->handleQuery('test', 20);
		$this->qm->handleQuery('Test ', 20);
		
		$in = $this->qm->getInformationNeed('Test');	
		$this->assertEquals($this->num_test_queries+3, $in['needExecutions']);
		
		// TODO - does not work yet as UTF-8
		// TODO umlaute etc.
		
		// umlaut test
		$in = $this->qm->getInformationNeed('Änderungswünsche');	
		$this->assertFalse($in);
		
		$this->qm->handleQuery('änderungswünsche', 20);
		$this->qm->handleQuery('änderungsWÜnsche', 20);
		
		$in = $this->qm->getInformationNeed('Änderungswünsche');	
		$this->assertEquals(2, $in['needExecutions']);
		
	}
	
	public function testHandleBrowse(){
		// TODO
		//$this->qm->handleBrowse($searchString, $sessionHash);
	}
	
	public function testHandleClick(){
		
		$in = $this->qm->getInformationNeed('Test');	
		$this->assertEquals($this->num_test_clicks, $in['resultClicks']);
		$pr = $this->qm->getPopularResults(10);
		$e = $pr[0]->getEntity();
		$this->assertEquals('http://www.test.de', $e['uri']);
		$this->assertEquals($this->num_testde_clicks, $pr[0]->getFreq());
		
		$this->qm->handleClick('Test', 'http://www.test.de', 1, '');

		$in = $this->qm->getInformationNeed('Test');	
		$this->assertEquals($this->num_test_clicks+1, $in['resultClicks']);
		$pr = $this->qm->getPopularResults(10);
		$e = $pr[0]->getEntity();
		$this->assertEquals('http://www.test.de', $e['uri']);
		$this->assertEquals($this->num_testde_clicks+1, $pr[0]->getFreq());
	}
	
	public function testGetUnsatisfiedQueriesWithFewHits(){
	}
	
	public function testGetUnsatisfiedQueriesWithManyDifferentClicks(){
	}
	
	public function testGetUnsatisfiedQueriesWithMuchBrowsing(){
	}
	
	public function testGetUnsatisfiedQueriesAggregations(){
	}
	
	
	
	public function testGetInformationNeed(){
	   	$in = $this->qm->getInformationNeed('Test'); // uppercase
		$this->assertEquals($this->num_test_queries, $in['needExecutions']);

	   	$in = $this->qm->getInformationNeed('test'); // lowercase
		$this->assertEquals($this->num_test_queries, $in['needExecutions']);

	   	$in = $this->qm->getInformationNeed('test '); // space
		$this->assertEquals($this->num_test_queries, $in['needExecutions']);

	   	// TODO extend e.g. umaut
	} 
 	
	public function testGetRelatedQueries(){
		$qStart = microtime(true); // TODO performance
	
		$all = $this->qm->getRelatedQueries('Test', 10, false);
		$this->assertEquals($this->num_diff_queries, count($all));

		$all = $this->qm->getRelatedQueries('test', 10, false);
		$this->assertEquals($this->num_diff_queries, count($all));
		
		$all = $this->qm->getRelatedQueries(' Test ', 10, false);
		$this->assertEquals($this->num_diff_queries, count($all));
		
		// TODO umlaaut
		
		$other = $this->qm->getRelatedQueries('Test', 10, true);
		$this->assertEquals($this->num_diff_queries-1, count($other));
	 	
		$qStop = microtime(true);
		$qTime = $qStop - $qStart;
		echo "\n RelatedQueries-Query-Time: " . $qTime;
	}
	
	public function testGetPopularQueries() {
		$pq = $this->qm->getPopularQueries(10);
		$this->assertEquals('test', $pq[0]->getName());
		$this->assertEquals($this->num_test_queries, $pq[0]->getFreq());
		$this->assertEquals($this->num_diff_queries, count($pq));
	}
	
	public function testGetPopularResults() {
		$pr = $this->qm->getPopularResults(10);
		$e = $pr[0]->getEntity();
		$this->assertEquals('http://www.test.de', $e['uri']);
		$this->assertEquals($this->num_testde_clicks, $pr[0]->getFreq());
		$this->assertEquals($this->num_diff_results, count($pr));
	}
	
	public function testGetPopularSources() {
		// TODO - there is only one repository currently, and this is not correctly stored!
		//$ps = $this->qm->getPopularSources(10);
		//echo 'PS SIZE: ' . count($ps);
		//print_r($ps);
	}
	
	public function testGetTags() {
		$tg = $this->qm->getTags('http://www.test.de');
		$this->assertEquals('test', $tg[0]->getName());
		$this->assertEquals(3, $tg[0]->getFreq());
		$this->assertEquals(1, count($tg));
	}
	
	public function testDropAndInitDb(){

		// table should exist due to preparation
		$this->assertTrue(WoogleMWUtil::tableExists('woogle_aggquery'));
		$this->qm->dropDb();	
		$this->assertFalse(WoogleMWUtil::tableExists('woogle_aggquery'));
		$this->qm->initDb();	
		$this->assertTrue(WoogleMWUtil::tableExists('woogle_aggquery'));
		
	}
	
	public function testNormalizeQueryString(){
		$this->assertEquals('hello world!', $this->qm->normalizeQueryString(' hello world! '));
		$this->assertEquals('hello world!', $this->qm->normalizeQueryString('hEllo WORld!'));
	}
	
	public function testGetHash(){
		$query = "hello";
		$this->assertEquals(md5($hello), WoogleQueryManagerNative::getHash($hello));
	}

	
	/*
	 *	Methods for testing database content (aggquery) 
	 *
	 */

	public function testNeedexec(){
		$in = $this->qm->getInformationNeed('test');
		$this->assertEquals($this->num_test_queries, $in['needExecutions']);
		
		// additional query
		$this->qm->handleQuery('test', 20);
		$in = $this->qm->getInformationNeed('test');
		$this->assertEquals($this->num_test_queries+1, $in['needExecutions']);
		
		// additional query
		$this->qm->handleQuery(' TesT', 20);
		$in = $this->qm->getInformationNeed('test');
		$this->assertEquals($this->num_test_queries+2, $in['needExecutions']);

		// browse
		// TODO think hash
		$this->qm->handleBrowse('test', '');
		$in = $this->qm->getInformationNeed('test');
		$this->assertEquals($this->num_test_queries+2, $in['needExecutions']);
		
	}
	
	public function testNeedexecViaQueryService(){
		$in = $this->qm->getInformationNeed('test');
		$this->assertEquals($this->num_test_queries, $in['needExecutions']);
		
		$values = self::getValues(); 
		
		// additional query
		$this->qs->executeKeywordQueryValues('test', 10, 0, $values, 'unittest');
		$in = $this->qm->getInformationNeed('test');
		$this->assertEquals($this->num_test_queries+1, $in['needExecutions']);
		
		// additional query
		$this->qs->executeKeywordQueryValues(' TesT', 10, 0, $values, 'unittest');
		$in = $this->qm->getInformationNeed('test');
		$this->assertEquals($this->num_test_queries+2, $in['needExecutions']);
		
		// browsing (should not increment query)
		global $wgRequest;
		$wgRequest->setVal('page', 10);
		
		$this->qs->executeKeywordQueryValues('test', 10, 10, $values, 'unittest');
		$in = $this->qm->getInformationNeed('test');
		$this->assertEquals($this->num_test_queries+2, $in['needExecutions']);
		
		
	}
	
	public function testNeedexecclick(){
		// base info
		$in = $this->qm->getInformationNeed('testxy');
		$this->assertFalse($in);
		
		// additional query
		$this->qm->handleQuery('testxy', 20);
		$in = $this->qm->getInformationNeed('testxy');
		$this->assertEquals(0+1, $in['needExecutionsWithoutResultClick']);
		$this->assertEquals(0, $in['resultClicks']);
		
		// additional query
		$this->qm->handleQuery('testxy', 20);
		$in = $this->qm->getInformationNeed('testxy');
		$this->assertEquals(0+2, $in['needExecutionsWithoutResultClick']);
		$this->assertEquals(0, $in['resultClicks']);
		
		// click base data
		$pr = $this->qm->getPopularResults(10);
		$e = $pr[0]->getEntity();
		$this->assertEquals('http://www.test.de', $e['uri']);
		$this->assertEquals($this->num_testde_clicks, $pr[0]->getFreq());
		
		// first click
		$this->qm->handleClick('testxy', 'http://www.test.de', 5, 'session1');
		
		$in = $this->qm->getInformationNeed('Testxy');	
		$this->assertEquals(0+1, $in['resultClicks']);
		$this->assertEquals(0+1, $in['needExecutionsWithoutResultClick']);
		$pr = $this->qm->getPopularResults(10);
		$e = $pr[0]->getEntity();
		$this->assertEquals('http://www.test.de', $e['uri']);
		$this->assertEquals($this->num_testde_clicks+1, $pr[0]->getFreq());
		
		// second click - same session
		$this->qm->handleClick('Testxy', 'http://www.test.de', 1, 'session1');
		
		$in = $this->qm->getInformationNeed('TestXY');	
		$this->assertEquals(0+2, $in['resultClicks']);
		$this->assertEquals(0+1, $in['needExecutionsWithoutResultClick']);
		$pr = $this->qm->getPopularResults(10);
		$e = $pr[0]->getEntity();
		$this->assertEquals('http://www.test.de', $e['uri']);
		$this->assertEquals($this->num_testde_clicks+2, $pr[0]->getFreq());

		// third click - new session
		$this->qm->handleClick('Testxy', 'http://www.test.de', 1, 'session2');
		
		$in = $this->qm->getInformationNeed('Testxy');	
		$this->assertEquals(0+3, $in['resultClicks']);
		$this->assertEquals(0, $in['needExecutionsWithoutResultClick']);
		$pr = $this->qm->getPopularResults(10);
		$e = $pr[0]->getEntity();
		$this->assertEquals('http://www.test.de', $e['uri']);
		$this->assertEquals($this->num_testde_clicks+3, $pr[0]->getFreq());
		
	}
	
	public function testRecency(){
		// TODO
	}
	
	public function testFirstLastDate(){
		
		// base data (no query yet)
		$in = $this->qm->getInformationNeed('firstlastdate');
		$this->assertFalse($in);
		
		// first entry
		$this->qm->handleQuery('firstlastdate', 20);
		$in = $this->qm->getInformationNeed('firstlastdate');
		$iDate = $in['firstDate'];
		$this->assertEquals($iDate, $in['lastDate']);
		
		// second entry
		sleep(1); // wait a second
		$this->qm->handleQuery('firstlastdate', 20);
		$in = $this->qm->getInformationNeed('firstlastdate');
		$sDate = $in['lastDate'];
		$this->assertEquals($iDate, $in['firstDate']);
		echo " -------- sDate: " . $sDate . " - iDate: " . $iDate;   // TODO rmv
		$this->assertTrue($sDate > $iDate);
		
		// third entry
		sleep(1); // wait a second
		$this->qm->handleQuery('firstlastdate', 20);
		$in = $this->qm->getInformationNeed('firstlastdate');
		$this->assertEquals($iDate, $in['firstDate']);
		$this->assertTrue($in['lastDate'] > $iDate);
		$this->assertTrue($in['lastDate'] > $sDate);
		
	}
	
	public function testResultHits(){
		// TODO done in other codeß
	}
	
	public function testQueryUsers(){
		
		// base data
		$in = $this->qm->getInformationNeed('test');
		$this->assertEquals(1, $in['sizeQueryUsers']);
		
		// get queryUsers manually
		$db = WoogleMWUtil::getDB();
		extract($db->tableNames('woogle_aggquery'));
		$searchString = WoogleQueryManagerNative::normalizeQueryString('test');
		$sql = 'SELECT * FROM ' . $woogle_aggquery . ' ' .
			'WHERE aggquery_qid = "' . mysql_real_escape_string(WoogleQueryManagerNative::getHash($searchString)) . '"';
	
		$res = $db->query($sql);
		if ($row = $db->fetchObject($res)) {
			$u = WoogleQueryManagerNative::getUsers($row->aggquery_queryusers);
		}
		$this->assertEquals(1, count($u));
		
		// first query - same user
		$this->qm->handleQuery('test', 20);
		$in = $this->qm->getInformationNeed('test');
		$this->assertEquals(1, $in['sizeQueryUsers']);
		$res = $db->query($sql);
		if ($row = $db->fetchObject($res)) {
			$u = WoogleQueryManagerNative::getUsers($row->aggquery_queryusers);
		}
		$this->assertEquals(1, count($u));
		
		// second query - second user
		global $wgUser;
		$wgUser = new MockUser('wikisysop');
		WoogleConfig::setPref('Privacy', 'RandomUserID', 'wikisysop_id');
		$this->qm->handleQuery('test', 20);
		$in = $this->qm->getInformationNeed('test');
		$this->assertEquals(2, $in['sizeQueryUsers']);
		$res = $db->query($sql);
		if ($row = $db->fetchObject($res)) {
			$u = WoogleQueryManagerNative::getUsers($row->aggquery_queryusers);
		}
		$this->assertEquals(2, count($u));
		
		// third query - third user
		global $wgUser;
		$wgUser = new MockUser('karlheinz');
		WoogleConfig::setPref('Privacy', 'RandomUserID', 'karlheinz_id');
		$this->qm->handleQuery('test', 20);
		$in = $this->qm->getInformationNeed('test');
		$this->assertEquals(3, $in['sizeQueryUsers']);
		$res = $db->query($sql);
		if ($row = $db->fetchObject($res)) {
			$u = WoogleQueryManagerNative::getUsers($row->aggquery_queryusers);
		}

		$this->assertEquals(3, count($u));
		$this->assertTrue(in_array('i_' . md5(WoogleUtil::getSecretKey() . '127.0.0.1'), $u)); // anonymized IP for anon
		$this->assertTrue(in_array('wikisysop_id', $u));
		$this->assertTrue(in_array('karlheinz_id', $u));
		
	}
	
	public function testNeedexecpages(){

		// base data
		$in = $this->qm->getInformationNeed('test');
		$this->assertEquals(0, $in['needExecutionsWithFurtherPages']);
		$this->assertEquals(1, $in['needExecutionsWithFurtherPagesAveragePages']);
		
		// query
		$this->qm->handleQuery('firstlastdate', 20);
		$in = $this->qm->getInformationNeed('test');
		$this->assertEquals(0, $in['needExecutionsWithFurtherPages']);
		$this->assertEquals(1, $in['needExecutionsWithFurtherPagesAveragePages']);
		
		// browse #1
		$this->qm->handleBrowse('test', 'session1');
		$in = $this->qm->getInformationNeed('test');
		$this->assertEquals(1, $in['needExecutionsWithFurtherPages']);
// TODO		$this->assertEquals(1, $in['needExecutionsWithFurtherPagesAveragePages']);
		
		// browse #2 - same session
		$this->qm->handleBrowse('test', 'session1');
		$in = $this->qm->getInformationNeed('test');
		$this->assertEquals(1, $in['needExecutionsWithFurtherPages']);
// TODO		$this->assertEquals(1, $in['needExecutionsWithFurtherPagesAveragePages']);
		
		// browse #3 - new session
		$this->qm->handleBrowse('test', 'session2');
		$in = $this->qm->getInformationNeed('test');
		$this->assertEquals(2, $in['needExecutionsWithFurtherPages']);
// TODO		$this->assertEquals(1, $in['needExecutionsWithFurtherPagesAveragePages']);
		
		// TODO session/define - avg pages?
		
	}
	
	public function testResultClicks(){
		// base info
		$in = $this->qm->getInformationNeed('test');
		$this->assertEquals($this->num_test_clicks, $in['resultClicks']);
		$this->assertEquals($this->num_diff_results, $in['uniqueClickedResults']);
		
		// first click - old result
		$this->qm->handleClick('Test', 'http://www.test.de', 1, '');
		$in = $this->qm->getInformationNeed('test');
		$this->assertEquals($this->num_test_clicks+1, $in['resultClicks']);
		$this->assertEquals($this->num_diff_results, $in['uniqueClickedResults']);

		// second click - new result
		$this->qm->handleClick('Test', 'http://www.test.at', 5, '');
		$in = $this->qm->getInformationNeed('test');
		$this->assertEquals($this->num_test_clicks+2, $in['resultClicks']);
		$this->assertEquals($this->num_diff_results+1, $in['uniqueClickedResults']);
		
	}
	
	/*
	 *	Methods for testing database content (aggclick) 
	 *
	 */
	
	public function testClicks(){
		// base data
		$tg = $this->qm->getTags('http://www.test.de');
		$this->assertEquals('test', $tg[0]->getName());
		$this->assertEquals(3, $tg[0]->getFreq());
		$this->assertEquals(1, count($tg));
		
		// first click
		$this->qm->handleClick('Test', 'http://www.test.de', 1, '');
		$tg = $this->qm->getTags('http://www.test.de');
		$this->assertEquals('test', $tg[0]->getName());
		$this->assertEquals(4, $tg[0]->getFreq());
		$this->assertEquals(1, count($tg));
		
		// second click
		$this->qm->handleClick('Test', 'http://www.test.de', 1, '');
		$tg = $this->qm->getTags('http://www.test.de');
		$this->assertEquals('test', $tg[0]->getName());
		$this->assertEquals(5, $tg[0]->getFreq());
		$this->assertEquals(1, count($tg));
		
		// third click (diff query)
		$this->qm->handleQuery('Test2', 20);
		$this->qm->handleClick('Test2', 'http://www.test.de', 1, '');
		$tg = $this->qm->getTags('http://www.test.de');
		echo "---- TAG " . print_r($tg, true);
		
		// ONLY FOR TESTING - TODO
		$db = WoogleMWUtil::getDB();
		extract($db->tableNames('woogle_aggclick'));
		$sql = 'SELECT * FROM ' . $woogle_aggclick;
		echo "SQL: " . $sql;
		$res = $db->query($sql);
		while ($row = $res->next()) echo 'ROWC ' . print_r($row, true);

		/*
		$db = WoogleMWUtil::getDB();
		extract($db->tableNames('woogle_aggclick'));
		$sql = 'SELECT * FROM ' . $woogle_aggquery;
		echo "SQL: " . $sql;
		$res = $db->query($sql);
		while ($row = $res->next()) echo 'ROWQ ' . print_r($row, true); 
		*/
		
		$this->assertEquals('test', $tg[0]->getName());
		$this->assertEquals(5, $tg[0]->getFreq());
		$this->assertEquals(2, count($tg));
		$this->assertEquals('test2', $tg[1]->getName());
		$this->assertEquals(1, $tg[1]->getFreq());
		
		// forth click (diff url)
		$this->qm->handleClick('Test', 'http://www.test.at', 1, '');
		$tg = $this->qm->getTags('http://www.test.at');
		$this->assertEquals('test', $tg[0]->getName());
		$this->assertEquals(1, $tg[0]->getFreq());
		$this->assertEquals(1, count($tg));
		
		// fifth click (diff url)
		$this->qm->handleClick('Test', 'http://www.test.at', 1, '');
		$tg = $this->qm->getTags('http://www.test.at');
		$this->assertEquals('test', $tg[0]->getName());
		$this->assertEquals(2, $tg[0]->getFreq());
		$this->assertEquals(1, count($tg));
	}
	
	public function testPosition(){
		
		$db = WoogleMWUtil::getDB();
		extract($db->tableNames('woogle_aggclick'));
		$searchString = WoogleQueryManagerNative::normalizeQueryString('test');
		$uri = 'http://www.test.de';
		$sql = 'SELECT * FROM ' . $woogle_aggclick . ' ' .
			'WHERE aggclick_qid = "' . mysql_real_escape_string(WoogleQueryManagerNative::getHash($searchString)) . '"' .
		    ' AND aggclick_subject = "' . $uri . '"';
		$res = $db->query($sql);
		if ($row = $db->fetchObject($res)) {
			$u = $row->aggclick_position;
		}
		$this->assertEquals(1, $u);
		
		// first click
		$this->qm->handleClick('Test', 'http://www.test.de', 5, '');
		$res = $db->query($sql);
		if ($row = $db->fetchObject($res)) {
			$u = $row->aggclick_position;
		}
		$this->assertEquals(5, $u);
		
		// second click
		$this->qm->handleClick('Test', 'http://www.test.de', 1, '');
		$res = $db->query($sql);
		if ($row = $db->fetchObject($res)) {
			$u = $row->aggclick_position;
		}
		$this->assertEquals(1, $u);
	}
	
	public function testClickFirstLastDate(){
		
		// sql
		$db = WoogleMWUtil::getDB();
		extract($db->tableNames('woogle_aggclick'));
		$searchString = WoogleQueryManagerNative::normalizeQueryString('firstlastclick');
		$uri = 'http://www.firstlastclick.de';
		$sql = 'SELECT * FROM ' . $woogle_aggclick . ' ' .
			'WHERE aggclick_qid = "' . mysql_real_escape_string(WoogleQueryManagerNative::getHash($searchString)) . '"' .
		    ' AND aggclick_subject = "' . $uri . '"';
		$first = '';
		$last = '';
		
		// first click
		$this->qm->handleClick('firstlastclick', 'http://www.firstlastclick.de', 1, '');
		$res = $db->query($sql);
		if ($row = $db->fetchObject($res)) {
			print_r($row);
			$first = $row->aggclick_firstdate;
			$last = $row->aggclick_lastdate;
		}
		$this->assertEquals($first, $last);
		$first_one = $first;
		
		// second click
		sleep(1); // wait a second
		$this->qm->handleClick('firstlastclick', 'http://www.firstlastclick.de', 1, '');
		$res = $db->query($sql);
		if ($row = $db->fetchObject($res)) {
			print_r($row);
			$first = $row->aggclick_firstdate;
			$last = $row->aggclick_lastdate;
		}
		
		echo "----- FIRST: " . $first_one . " / LAST: " . $last;  // TODO rmv
		$this->assertEquals($first_one, $first);
		$this->assertTrue($last > $first_one);
		$last_two = $last;
		
		// thrid click
		sleep(1); // wait a second
		$this->qm->handleClick('firstlastclick', 'http://www.firstlastclick.de', 1, '');
		$res = $db->query($sql);
		if ($row = $db->fetchObject($res)) {
			$first = $row->aggclick_firstdate;
			$last = $row->aggclick_lastdate;
		}
		$this->assertEquals($first_one, $first);
		$this->assertTrue($last > $first);
		$this->assertTrue($last > $last_two);
		
	}
}



?>
